/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/*
 * Characterization of buzzers
 *
 * This demo will calibrate the min,max duty-cycles for a buzzer.
 *
 * The demo start with DutyCycle=0% and rises it up.
 * When the user feels the vibration, she has to press the User Button to mark
 * the MinDutyCylce.
 *
 * NOTE: If you want additional information at runtime, #define HL_DEBUG
 * (either in hapticLib.h or in the gcc command line).
 * Doing so, the USART will be enabled and the green led will show 
 * the PWM sent to the haptic device.
 *
 * Guardati Leonardo
 * version 0.7
 * 2012
 *
 */

#include "stm32f10x.h"
#include "hapticLib.h"

int main(void)
{
	/* 
	 * Here the System Clock must be already configured.
   	 * Make sure the startup code (vector.c is the default) calls SystemInit()
   	 * before branching here to main().
   	 */

	EXTI_InitTypeDef InitStruct;

	haptor_desc *myHaptor;
	user_param   myParams;

	myHaptor = hl_configure(24000000,10,1);

	// User Button Event Configuration 
	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource0);
	GPIO_EventOutputConfig(GPIO_PortSourceGPIOA, GPIO_PinSource0);


	InitStruct.EXTI_Line    = EXTI_Line0;
	InitStruct.EXTI_Mode    = EXTI_Mode_Interrupt;
	InitStruct.EXTI_Trigger = EXTI_Trigger_Rising;
	InitStruct.EXTI_LineCmd = ENABLE;

	EXTI_Init(&InitStruct);

#ifdef HL_DEBUG
	send_string("\r\n\r\n\0");
	send_string("\t\tWelcome to HapticCalibrator!!\r\n\r\n\0");
	send_string("Now we will calibrate the System's Haptors...\r\n\0");
	send_string("As soon as you CLEARLY feel the vibration, hit the User Button!!! ( Be quick! ;) )\r\n\r\n\r\n\0");
#endif

	Delay(1500);

#ifdef HL_DEBUG
	send_string("Wear the Haptor and when you feel ready, press the User Button.\r\n\0");
#endif

	while ( EXTI_GetFlagStatus(EXTI_Line0) != SET ) { __WFI(); } ;

	EXTI_ClearFlag(EXTI_Line0);

	myParams.constant.constant = 0x0000;
	hl_startPattern( hl_addHaptor(myHaptor, hl_initPattern(Constant, &myParams) ) );

	while ( myParams.constant.constant < 0xff00 )
	{
#ifdef HL_DEBUG
		send_string("Duty Cycle: \0");
		send_int(myParams.constant.constant);
		send_string("\r\n\0");
#endif
		myParams.constant.constant += 10;

		Delay(1);

		if( EXTI_GetFlagStatus(EXTI_Line0) == SET )
		{
			EXTI_ClearFlag(EXTI_Line0); 
#ifdef HL_DEBUG
			send_string("GOOD! We found the minimum dutyCycle!\r\n\0");
#endif
			GPIO_WriteBit(GPIOC, GPIO_Pin_8, Bit_SET);
			break;
		}
	}

	if( myParams.constant.constant >= 0xff00 )
	{
#ifdef HL_DEBUG
		send_string("BAD!! We reached almost the 100\% DutyCycle and you didn't press.\r\n\0");
		send_string("Make sure your Haptor isn't broken!\r\n\0");
#endif
	} 
	else 
	{
#ifdef HL_DEBUG
		send_string("Now setting the minimum activating DutyCycle to: \0");
		send_int(myParams.constant.constant);
		send_string("\r\n\0");
#endif

		myHaptor->min_duty = myParams.constant.constant;
	}

	hl_stopPattern(myHaptor->activePattern);

#ifdef HL_DEBUG
	send_string("\r\nGoodbye....going to sleep...\r\n\0");
#endif

	while(1) { __WFE(); }
}
