
/**
  * @file    LIS3LV02DL.c
  * @author  Michele Magno MicreLab Lab
  * @version V0.1.0
  * @date    21/02/2012
  * @brief   This file provides a set of functions needed to manage the
  *          communication between STM32 I2C master and LIS3LV02DL I2C slave.
  * @details
  *
  * 
  *
  * COPYRIGHT 2012 MicreLab Universit of Bologna
  */


#include "HAL_LIS3LV02DL.h"
#include "lis302dl.h"
#include "stm32f10x_conf.h"

//#include "global_header.h"

 /**
* @defgroup LIS3LV02DL
* @{
*/

/** @defgroup LIS3LV02DL_I2C_Function
* @{
*/

/**
* @brief  Initializes the I2C peripheral used to drive the LIS3LV02DL
* @param  None
* @retval None
*/
void LIS3LV02DL_I2C_Init(void)
{
  I2C_InitTypeDef  I2C_InitStructure;
  GPIO_InitTypeDef GPIO_InitStructure;

  I2C_DeInit(LIS_I2C);

  /* Enable I2C, GPIO and AFIO clocks */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);
  RCC_APB1PeriphClockCmd(LIS_I2C_RCC_Periph, ENABLE);
  RCC_APB2PeriphClockCmd(LIS_I2C_RCC_Port, ENABLE);

  /* Configure I2C pins: SCL and SDA */
  GPIO_InitStructure.GPIO_Pin =  LIS_I2C_SCL_Pin | LIS_I2C_SDA_Pin;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
  GPIO_Init(LIS_I2C_Port, &GPIO_InitStructure);

  /* Pins remapping */
  //GPIO_PinRemapConfig(LIS_I2C_REMAP, ENABLE);

  /* I2C configuration */
  I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
  I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
  I2C_InitStructure.I2C_OwnAddress1 = 0x00;
  I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
  I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
  I2C_InitStructure.I2C_ClockSpeed = LIS_I2C_Speed;

  /* I2C Peripheral Enable */
  I2C_Cmd(LIS_I2C, ENABLE);

 	I2C1->CR1 = 0x8000;

 	I2C1->CR1 = 0x0000;

 // /* I2C Peripheral Enable */
  I2C_Cmd(LIS_I2C, ENABLE);

  /* Apply I2C configuration after enabling it */
  I2C_Init(LIS_I2C, &I2C_InitStructure);


}

/**
* @brief  Reads a block of data from the LIS3LV02DL.
* @param  slAddr  : slave address LIS_A_I2C_ADDRESS
* @param  pBuffer : pointer to the buffer that receives the data read from the LIS3LV02DL.
* @param  ReadAddr : LIS3LV02DL's internal address to read from.
* @param  NumByteToRead : number of bytes to read from the LIS3LV02DL
* @retval None
*/

void LIS3LV02DL_I2C_BufferRead(u8 slAddr, u8* pBuffer, u8 ReadAddr, u16 NumByteToRead)
{
  /* While the bus is busy */
  while(I2C_GetFlagStatus(LIS_I2C, I2C_FLAG_BUSY));

  /* Send START condition */
  I2C_GenerateSTART(LIS_I2C, ENABLE);

  /* Test on EV5 and clear it */
  while(!I2C_CheckEvent(LIS_I2C, I2C_EVENT_MASTER_MODE_SELECT));

  /* Send LIS3LV02DL address for write */
  I2C_Send7bitAddress(LIS_I2C, slAddr, I2C_Direction_Transmitter);

  /* Test on EV6 and clear it */
  while(!I2C_CheckEvent(LIS_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));

  /* Clear EV6 by setting again the PE bit */
  I2C_Cmd(LIS_I2C, ENABLE);

  /* Send the LIS3LV02DL's internal address to write to */
  I2C_SendData(LIS_I2C, ReadAddr);

  /* Test on EV8 and clear it */
  while(!I2C_CheckEvent(LIS_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  /* Send STRAT condition a second time */
  I2C_GenerateSTART(LIS_I2C, ENABLE);

  /* Test on EV5 and clear it */
  while(!I2C_CheckEvent(LIS_I2C, I2C_EVENT_MASTER_MODE_SELECT));

  /* Send LIS3LV02DL address for read */
  I2C_Send7bitAddress(LIS_I2C, slAddr, I2C_Direction_Receiver);

  /* Test on EV6 and clear it */
  while(!I2C_CheckEvent(LIS_I2C, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));

  /* While there is data to be read */
  while(NumByteToRead)
  {
    if(NumByteToRead == 1)
    {
      /* Disable Acknowledgement */
      I2C_AcknowledgeConfig(LIS_I2C, DISABLE);

      /* Send STOP Condition */
      I2C_GenerateSTOP(LIS_I2C, ENABLE);
    }

    /* Test on EV7 and clear it */
    if(I2C_CheckEvent(LIS_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED))
    {
      /* Read a byte from the LIS3LV02DL */
      *pBuffer = I2C_ReceiveData(LIS_I2C);

      /* Point to the next location where the byte read will be saved */
      pBuffer++;

      /* Decrement the read bytes counter */
      NumByteToRead--;
    }
  }

  /* Enable Acknowledgement to be ready for another reception */
  I2C_AcknowledgeConfig(LIS_I2C, ENABLE);
}
/**
 * @}
 */ /* end of group LIS3LV02DL_I2C_Function */



/**
* @brief  Writes one byte to the  LIS3LV02DL.
* @param  slAddr : slave address LSM_A_I2C_ADDRESS
* @param  pBuffer : pointer to the buffer  containing the data to be written to the LIS3LV02DL.
* @param  WriteAddr : address of the register in which the data will be written
* @retval None
*/
void LIS3LV02DL_I2C_ByteWrite(u8 slAddr, u8* pBuffer, u8 WriteAddr)
{
  /* Send START condition */
  I2C_GenerateSTART(LIS_I2C, ENABLE);

  /* Test on EV5 and clear it */
  while(!I2C_CheckEvent(LIS_I2C, I2C_EVENT_MASTER_MODE_SELECT));

  /* Send LIS3LV02DL address for write */
  I2C_Send7bitAddress(LIS_I2C, slAddr, I2C_Direction_Transmitter);

  /* Test on EV6 and clear it */
  while(!I2C_CheckEvent(LIS_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));

  /* Send the LIS3LV02DL's internal address to write to */
  I2C_SendData(LIS_I2C, WriteAddr);

  /* Test on EV8 and clear it */
  while(!I2C_CheckEvent(LIS_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  /* Send the byte to be written */
  I2C_SendData(LIS_I2C, *pBuffer);

  /* Test on EV8 and clear it */
  while(!I2C_CheckEvent(LIS_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  /* Send STOP condition */
  I2C_GenerateSTOP(LIS_I2C, ENABLE);
}



void LIS3LV02DL_Acc_Config (LIS_Acc_ConfigTypeDef *LIS_Acc_Config_Struct)
{

	  u8 CRTL1 = 0x00;
	  u8 CRTL2  =  0x00;


	  CRTL1 |= (u8) (LIS_Acc_Config_Struct->Power_Mode | LIS_Acc_Config_Struct->ODR| LIS_Acc_Config_Struct->Axes_Enable);

	  CRTL2 |= (u8) (LIS_Acc_Config_Struct->FS | LIS_Acc_Config_Struct->Data_Update| LIS_Acc_Config_Struct->Endianess | LIS_Acc_Config_Struct->Interrupt | LIS_Acc_Config_Struct->Data_Ready | LIS_Acc_Config_Struct->Serial_Mode | LIS_Acc_Config_Struct->Data_Alignment);

	  LIS3LV02DL_I2C_ByteWrite(LIS_A_I2C_ADDRESS, &CRTL1, LIS_A_CTRL_REG1_ADDR);
	  LIS3LV02DL_I2C_ByteWrite(LIS_A_I2C_ADDRESS, &CRTL2, LIS_A_CTRL_REG2_ADDR);

//  u8 tmp;
//
//  tmp = 3<<6 | 1<<4 | 7;	// acceso, decimazione di 128, no self test, tre assi abilitati
//  LIS3LV02DL_I2C_ByteWrite (LSM_A_I2C_ADDRESS, &tmp, LSM_A_CTRL_REG1_ADDR);
////	tmp = 1<<6;			// 2g fondo scala, Block data update, Little-endian, no reboot, no IRQ, right alignment
//	tmp = 1<<6 | 1;			// 2g fondo scala, Block data update, Little-endian, no reboot, no IRQ, right alignment, DAS
//  LIS3LV02DL_I2C_ByteWrite (LSM_A_I2C_ADDRESS, &tmp, LSM_A_CTRL_REG2_ADDR);
//	tmp = 0;
//  LIS3LV02DL_I2C_ByteWrite (LSM_A_I2C_ADDRESS, &tmp, LSM_A_CTRL_REG3_ADDR);
}

void LIS3LV02DL_Acc_Filter_Config (LIS_Acc_Filter_ConfigTypeDef *LIS_Acc_Filter_Config_Struct)
{
	u8 CRTL3 = 0x00;


	CRTL3 |= (u8) (LIS_Acc_Filter_Config_Struct->CLK_selsction | LIS_Acc_Filter_Config_Struct->HPDD | LIS_Acc_Filter_Config_Struct->HPFF | LIS_Acc_Filter_Config_Struct->HPF_Enable | LIS_Acc_Filter_Config_Struct->HPF_Frequency);

	LIS3LV02DL_I2C_ByteWrite(LIS_A_I2C_ADDRESS, &CRTL3, LIS_A_CTRL_REG3_ADDR);

}

void LIS3LV02DL_Acc_FullScale_Cmd(u8 FS_value)
{
	  u8 tmpreg;
	  LIS3LV02DL_I2C_BufferRead(LIS_A_I2C_ADDRESS, &tmpreg, LIS_A_CTRL_REG2_ADDR, 1);
	  tmpreg &= 0x7F;
	  tmpreg |= FS_value;
	  LIS3LV02DL_I2C_ByteWrite(LIS_A_I2C_ADDRESS,&tmpreg, LIS_A_CTRL_REG2_ADDR);

}


void LIS3LV02DL_Acc_DataRate_Cmd(u8 DataRateValue)
{
	  u8 tmpreg;
	  LIS3LV02DL_I2C_BufferRead(LIS_A_I2C_ADDRESS, &tmpreg, LIS_A_CTRL_REG1_ADDR, 1);
	  tmpreg &= 0xCF;
	  tmpreg |= DataRateValue;
	  LIS3LV02DL_I2C_ByteWrite(LIS_A_I2C_ADDRESS,&tmpreg, LIS_A_CTRL_REG1_ADDR);

}


void LIS3LV02DL_Acc_Reboot_Cmd(void)
{
	  u8 tmpreg;
	  LIS3LV02DL_I2C_BufferRead(LIS_A_I2C_ADDRESS, &tmpreg, LIS_A_CTRL_REG2_ADDR, 1);
	  tmpreg |= 0x10;
	  LIS3LV02DL_I2C_ByteWrite(LIS_A_I2C_ADDRESS,&tmpreg, LIS_A_CTRL_REG2_ADDR);

}

//void LIS_Acc_Read_OutReg(u8* out)
//{
//  LIS3LV02DL_I2C_BufferRead (LIS_A_I2C_ADDRESS, out, (LIS_A_OUT_X_L_ADDR | 0x80), 6);
//}

void LIS3LV02DL_Acc_Read_Buffer (u8* out)
{
	//u8* myptr = (u8*) out;
	LIS3LV02DL_I2C_BufferRead (LIS_A_I2C_ADDRESS, out, LIS_A_OUT_X_L_ADDR|0x80, 6);
}



void LIS3LV02DL_Acc_Read_RawData (s16* out, char direction)
{
	  u8 buffer[6];
	  u8 crtl2;
		u8 i = 0;

	  LIS3LV02DL_I2C_BufferRead(LIS_A_I2C_ADDRESS, &crtl2, LIS_A_CTRL_REG2_ADDR, 1);

	  LIS3LV02DL_I2C_BufferRead(LIS_A_I2C_ADDRESS, buffer, direction|0x80, 6);

	  /* check in the control register2 the data alignment*/
	  if(!(crtl2 & 0x20))
	  {
	    for(i=0; i<3; i++)
	    {
	      out[i]=(s16)(((u16)buffer[2*i+1] << 8) + buffer[2*i]);
	    }
	  }
	  else
	  {
	    for(i=0; i<3; i++)
	      out[i]=((s16)((u16)buffer[2*i] << 8) + buffer[2*i+1]);
	  }
}

void LIS3LV02DL_Acc_Read_Data(float* out)
{
	s16 tmp[3];
	LIS3LV02DL_Acc_Read_RawData (tmp, LIS_A_OUT_X_L_ADDR);

	out[0] = (float)tmp[0] * LIS_Acc_Coeff_2g;
	out[1] = (float)tmp[1] * LIS_Acc_Coeff_2g;
	out[2] = (float)tmp[2] * LIS_Acc_Coeff_2g;

}

//void LIS3LV02DL_Acc_Calibration(s16* raw_data, float* out)
//{
//	float data[3];
//    data[0]=(float)raw_data[0];
//	data[1]=(float)raw_data[1];
//	data[2]=(float)raw_data[2];
//
////	//Mper(ACC_Gain, data, data_out, 3, 3, 1, 3, 2);
////
////
////	data_out[0]= (Acc_Calibr*(data_out[0] + Acc_Offset_x));
////	data_out[1]= (Acc_Calibr*(data_out[1] + Acc_Offset_y));
////	data_out[2]= (Acc_Calibr*(data_out[2] + Acc_Offset_z));
//
//}

void LIS3LV02DL_Acc_Init (void)
{
	LIS_Acc_ConfigTypeDef  LIS_Acc_InitStructure;
	LIS_Acc_Filter_ConfigTypeDef LIS_Acc_FilterStructure;

	LIS_Acc_InitStructure.Power_Mode = LIS_Acc_PowerOn;
	LIS_Acc_InitStructure.ODR = LIS_Acc_ODR_640;
	LIS_Acc_InitStructure.Axes_Enable= LIS_Acc_XYZEN;
	LIS_Acc_InitStructure.FS = LIS_Acc_FS_2;
	LIS_Acc_InitStructure.Data_Update = LIS_Acc_BDU_Continuos;
	LIS_Acc_InitStructure.Endianess=LIS_Acc_Little_Endian;
	LIS_Acc_InitStructure.Interrupt=LIS_Acc_Int_Disable;
	LIS_Acc_InitStructure.Data_Ready=LIS_Acc_DRDY_Disable;
	LIS_Acc_InitStructure.Serial_Mode=LIS_Acc_SPI_4wire;
	LIS_Acc_InitStructure.Data_Alignment=LIS_Acc_Data_Alignment_16;

	LIS_Acc_FilterStructure.CLK_selsction=LIS_Acc_CLK_INT;
	LIS_Acc_FilterStructure.HPF_Enable=LIS_Acc_Filter_Disable;
	LIS_Acc_FilterStructure.HPDD=LIS_Acc_HPDD_Disable;
	LIS_Acc_FilterStructure.HPFF=LIS_Acc_HPFF_Disable;
	LIS_Acc_FilterStructure.HPF_Frequency=LIS_Acc_Filter_HPc512;

	LIS3LV02DL_Acc_Config(&LIS_Acc_InitStructure);
	LIS3LV02DL_Acc_Filter_Config(&LIS_Acc_FilterStructure);


//	u8 id;

//	LIS3LV02DL_Acc_Config ();
//  	LIS3LV02DL_I2C_BufferRead (LIS_A_I2C_ADDRESS, &id, LIS_A_WHO_AM_I, 1);
	//assert (id == 0x3A);
}
