/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/*
 * Level Demo
 *
 * This application will act as a spirit level device.
 * The haptor(s) are activated when the tilt of the surface
 * holding the device changes.
 *
 * The pattern used is the ConstantPattern.
 *
 * Hardware NOTE:
 *   LIS302DL digital interface is set to I2C
 *   On the main board (STM32VLDISCOVERY) the I2C1 peripheral
 *   is used, as the pins don't conflict with other devices.
 *
 *   Here the setting for I2C1 device:
 *  - APB1:
 *    + GPIOB:
 *      - PB6: I2C1(SCL)
 *      - PB7: SPI2(SDA)
 *
 * Leonardo Guardati
 *
 * version 0.7
 * 2012
 *
 */

#include "lis302dl.h"
 
#include "stm32f10x.h"

#include "hapticLib.h"

enum levelHaptors {
	Up              = 0,
	Right           = 1,
	Down            = 2,
	Left            = 3,

	NumLevelHaptors = 4
};

int main(void)
{
  	/* 
	 * Here the System Clock must be already configured.
   	 * Make sure the startup code (vector.c is the default) calls SystemInit()
   	 * before branching here to main().
   	 */

	haptor_desc *haptors;
	user_param   levelParams[NumLevelHaptors];
	uint8_t      i=0;
	int8_t       x,y,z;
	int16_t	     tmp;


#ifdef HL_DEBUG
//	send_string("HapticLevel: Welcome \r\n\0");
#endif

	// Haptors Initialization
	// Initialize the Haptic System (setting the PWM global period in Hz)
	haptors = hl_configure(24000000,10, NumLevelHaptors);


	Delay(500);
	// I2C1 peripheral Initialization
	LIS3LV02DL_I2C_Init();
	Delay(500);

	// LIS302DL Sensor Initialization
	LIS3LV02DL_Acc_Init();
	Delay(500);

	// Start each haptor
	for( i=0 ; i<NumLevelHaptors ; i++)
	{

		levelParams[i].constant.constant = 0;

		if(    hl_startPattern(
				     hl_addHaptor(  &haptors[i],
				    		        hl_initPattern(Constant,&levelParams[i]) ) )    )
		{
#ifdef HL_DEBUG
			send_string("HapticLevel: Problem Starting Haptor \0");
			send_int(i);
			send_string(".\r\n\0");
#endif
		}
	}

	while(1) {

		// Get X-axis value
		LIS3LV02DL_Acc_Read_RawData(&tmp,LIS_A_OUT_X_L_ADDR);
		x = (int8_t) (tmp >> 8);

		// Get Y-axis value
		LIS3LV02DL_Acc_Read_RawData(&tmp,LIS_A_OUT_Y_L_ADDR);
		y = (int8_t) (tmp >> 8);

		// Get Z-axis value
		LIS3LV02DL_Acc_Read_RawData(&tmp,LIS_A_OUT_Z_L_ADDR);
		z = (int8_t) (tmp >> 8);

#ifdef HL_DEBUG
		send_string("HapticLevel: X-Value \0");
		send_int(x);
		send_string(".\r\n\0");
#endif


		// positive orientation
		if( z >= 0 ) {
			if( x >= 0 ) {
				levelParams[Up].constant.constant     = 4096*x;
				levelParams[Down].constant.constant   = 0;
			}
			else {
				levelParams[Down].constant.constant   = -4096*x;
				levelParams[Up].constant.constant     = 0;
			}
			if( y >= 0 ) {
				levelParams[Left].constant.constant   = 4096*y;
				levelParams[Right].constant.constant  = 0;
			}
			else {
				levelParams[Right].constant.constant  = -4096*y;
				levelParams[Left].constant.constant   = 0;
			}
		}
		else { // negative orientation
			if( x <= 0 ) {
				levelParams[Up].constant.constant     = -4096*x;
				levelParams[Down].constant.constant   = 0;
			}
			else {
				levelParams[Down].constant.constant   = 4096*x;
				levelParams[Up].constant.constant     = 0;
			}
			if( y <= 0 ) {
				levelParams[Left].constant.constant   = -4096*y;
				levelParams[Right].constant.constant  = 0;
			}
			else {
				levelParams[Right].constant.constant  = 4096*y;
				levelParams[Left].constant.constant   = 0;
			}
		}

		// Sleep
		Delay(100);

	}

	while(1) { __WFE(); }
}
