/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file hapticLib.c
 *
 * @brief **User API Module** definitions
 *
 * This file is part of the **User API Module** of HapticLib.
 *
 * The code inside this module is meant to be platform independent.
 *
 * The functions defined in hapticLib.c are the only ones the
 * library user should need to call for his haptic applications.
 *
 * @note The only exception is the Delay() function which now reside
 *       in the **Platform Specific Module**.
 *
 * @see Please refer to the [Architecture](@ref hl_architecture)
 *      page for a complete overview of HapticLib structure.
 *
 * @see Please refer to the [Developer Guide](@ref dev_guide) for
 *      the internals of HapticLib's Modules interactions.
 *
 * @author Leonardo Guardati
 * @version 0.7
 * @date 2012
 *
 */

#define HL_SYSTEM_FILE

#include "hapticLib.h"

extern pattern_initiator patternMap[MAX_HAPTORS];

/**
 * @brief 	Global variable to describe the haptic system
 *
 * The Library uses this global variable to describe the system
 * and to keep track of the haptic devices' status and active
 * patterns' progress at any time.
 *
 * @note The library user doesn't need to access this variable.
 *
 * The SystemDesc variable holds a lot of informations, and its
 * members allow the system to access almost all the informations
 * needed.
 *
 * @see Please refer to the [Developer Guide](@ref dev_guide) page
 *      for details.
 *
 */
system_desc SystemDesc;

/**
 * @brief **API exposed to the user** to setup the system.
 *
 * Here the global [SystemDesc](@ref SystemDesc) variable is
 * initialized. Then all the system peripheral get configured based
 * on the high level informations passed by the user.
 *
 * @param[in] pwmFreq 	This is a `uint32_t` used to specify the
 * 						Frequency of the haptors control PWM
 * 						signals. Expressed in Hz.
 * @param[in] samplesDelay This is a `uint8_t` used to specify the
 * 						   delay (in ms) between successive haptors
 * 						   control PWM signals updates.
 *
 * @note Please refer to the
 *       [Platform Specific Module](@ref low_level)
 *       implementations of Timers and PWM generations to be sure of
 *       the meaning of these parameters.
 *
 * @param[in] numHaptors This is a `uint8_t` to specify the actual
 * 						 number of haptors the application will use.
 *
 * @note It is an important feature to be able to specify a minor
 *       number of devices to use instead of the hardware defined
 *       MAX_HAPTORS. Doing so, not only the code will be faster and
 *       the memory footprint smaller (todo), but also the
 *       resources that drive the unused haptors will be available
 *       to the application for other uses. (todo)
 *
 * @return `haptor_desc *` A reference to the array of the configured
 *         haptor is returned to the application.
 *
 * @retval NULL If the reference is NULL, an error has occurred.
 *
 * @todo Add input validation on pwm_freq and samples_delay passed
 * 	      parameters.
 */
haptor_desc *hl_configure(uint32_t pwmFreq, uint8_t samplesDelay, uint8_t numHaptors) {

	uint8_t i = 0;

	// Input validation MISSING

	RCC_Configuration();
	GPIO_Configuration();

#ifdef HL_DEBUG
	USART_Configuration();
#endif

	if( numHaptors > MAX_HAPTORS )
	{
#ifdef HL_DEBUG
		send_string("hl_configure(): This platform doesn't support \0");
		send_int( numHaptors );
		send_string(" haptors. (MAX=\0");
		send_int( MAX_HAPTORS );
		send_string(".\r\n\0");
#endif
		return NULL;
	}

	SystemDesc.num_haptors   = numHaptors;

	SystemDesc.pwm_freq      = pwmFreq;
	SystemDesc.samples_delay = samplesDelay;

	for( i=0 ; i < SystemDesc.num_haptors ; i++ )
		SystemDesc.patterns[i].name = Null;

	// todo clarify the interface with Platform specific module
	//      for haptor initialization.
	TIM_Configuration(pwmFreq);

	// todo this is the temporary hardware activation...
	// until a clear interface is defined.
	for( i=0 ; i< SystemDesc.num_haptors ; i++ )
	{
		SystemDesc.haptors[i].id = i+1;  //temporary

		if( TIM_Channel_Enable(i+1) )
		{
#ifdef HL_DEBUG
			send_string("hl_configure(): Problem activating Hardware PWM!\r\n\0");
#endif
			return NULL;
		}
	}

	for( i=0 ; i < SystemDesc.num_haptors ; i++ )
	{
		SystemDesc.haptors[i].min_duty      = 0x0000;
		SystemDesc.haptors[i].max_duty      = 0xFFFF;
		SystemDesc.haptors[i].activePattern = NULL;
		SystemDesc.haptors[i].nextHaptor    = NULL;
	}

	return SystemDesc.haptors;
}

/**
 * @brief **API exposed to the user** to initialize a new pattern.
 *
 * Calling hl_initPattern(), the system will check for available
 * resources and then create the new pattern descriptor returning
 * its reference to the application who called.
 *
 * The pattern repository offer a wide selection of haptic feedback
 * patterns.
 *
 * The user must read the documentation of the patterns she wants to
 * use, in order to know if and what kind of **user parameters** the
 * pattern needs.
 *
 * @param[in] patternName This is an integer value (encoded with the
 * 						  @ref pattern_name enumeration) used to
 * 						  select the desired pattern.
 *
 * @param[out] userParams This is a pointer to the user_param
 * 						  structure the application must allocate to
 * 						  store the parameters needed by the pattern.
 *
 * @note user_param is a union type definition. With this type the
 *       compiler can enforce type checking at compile time to ensure
 *       a valid argument is passed. (earlier version of HapticLib
 *       used a `void *` to pass the user parameters).
 *
 * @return `pattern_desc *` A pointer to the pattern descriptor is
 *                          returned to the application.
 *                          The application will use this descriptor
 *                          to make successive operations related to
 *                          this pattern (like, adding haptors to it,
 *                          starting it, stopping it).
 *
 * @retval NULL If the reference is NULL, an error has occurred.
 *
 */
pattern_desc *hl_initPattern(pattern_name patternName, user_param *userParams)
{
	uint8_t i=0;

	// Input validation
	if( Null >= patternName || patternName >= Num_Patterns_Available)
	{
#ifdef HL_DEBUG
		send_string("hl_initPattern(): patternName is Invalid!\r\n\0");
#endif
		return NULL;
	}

	// Search an available pattern slot and initialize it.
	for( i=0 ; i < SystemDesc.num_haptors ; i++ )
	{
		if( SystemDesc.patterns[i].name == Null )
		{
			SystemDesc.patterns[i].name             = patternName;
			SystemDesc.patterns[i].userParams       = userParams;
			SystemDesc.patterns[i].continuator      = NULL;
			SystemDesc.patterns[i].activeHaptorList = NULL;

			return &SystemDesc.patterns[i];
		}
	}

	// Else error.
#ifdef HL_DEBUG
	send_string("hl_initPattern(): No more Pattern can be created, every haptor is busy!\r\n\0");
#endif
	return NULL;

}

/**
 * @brief **API exposed to the user** to add an haptor to a pattern.
 *
 * The user needs to call hl_addHaptor() to let an already
 * initialized pattern know which haptors to activate with its
 * logic.
 *
 * If the haptor is good (and free) and the pattern is good,
 * hl_addHaptor() will link the pattern to the haptor and,
 * vice-versa. If the pattern has already haptors linked to it,
 * the `newHaptor` will be added to the list formed by all the
 * haptors already attached to the pattern.
 *
 * @param[out] newHaptor This is the pointer to the haptor to be
 * 					 	 added to the list of attached haptors
 * 					 	 of the pattern.
 *
 * @param[out] pattern This is the pointer to a `pattern_desc`
 * 					   returned by a precedent hl_initPattern()
 * 					   call.
 *
 * @return `pattern_desc *` hl_addHaptor() returns the same
 * 							pointer passed as argument by
 * 							the caller.
 *
 * @retval NULL If the reference is NULL, an error has occurred.
 *
 * @note Returning the `pattern_desc *` of the pattern, it is
 *       possible to nest the calling sequence that lead to a
 *       single haptor pattern activation.
 * ~~~~~~~~
 *		hl_startPattern( hl_addHaptor( &myHaptors[2], hl_initPattern(Test,NULL) ) );
 * ~~~~~~~~
 *
 *
 */
pattern_desc *hl_addHaptor(haptor_desc *newHaptor, pattern_desc *pattern)
{
	haptor_desc *tmp = NULL;

	// Input validation: haptor
	if( ( newHaptor == NULL ) || ( newHaptor->activePattern != NULL ) )
	{
#ifdef HL_DEBUG
		send_string("hl_addHaptor(): haptor invalid or busy with another pattern!\r\n\0");
#endif
		return NULL;
	}

	// Input validation: pattern
	if( ( pattern == NULL ) || \
		( Null >= pattern->name ) || ( pattern->name >= Num_Patterns_Available))
	{
#ifdef HL_DEBUG
		send_string("hl_addHaptor(): This pattern is not valid or not initialized!\r\n\0");
#endif
		return NULL;
	}

	// Add the haptor to the pattern's haptor list!
	tmp = pattern->activeHaptorList;

	if( pattern->activeHaptorList == NULL )
		pattern->activeHaptorList = newHaptor;
	else
	{
		tmp = pattern->activeHaptorList;
		while( tmp->nextHaptor )
				tmp = tmp->nextHaptor;
		tmp->nextHaptor = newHaptor;
	}

	newHaptor->activePattern = pattern;
	newHaptor->nextHaptor    = NULL;
	
	return pattern;
}

/**
 * @brief **API exposed to the user** to send Haptic Feedback.
 *
 * hl_startPattern() is used to activate an already initialized
 * pattern with a set of haptors attached to it.
 *
 * After some validation checks on the pattern passed (it must
 * be ready to be activated), hl_startPattern() will call the
 * pattern's initiator code.
 *
 * @see To learn about how a pattern generator works, please read
 *      the [Developer Page](@ref dev_guide).
 *
 * hl_startPattern() uses the @ref patternMap array to be able
 * to call the right initiator.
 *
 * @note You cannot use the same pattern descriptor to activate
 *       the pattern simultaneously on different haptors.\n
 *       If the pattern is multi haptor, you have to call
 *       addHaptor() for each haptor to add to the same pattern.\n
 *       If the pattern is single-haptor, and you want multiple
 *       instances running simultaneously, you need to initialize
 *       a new pattern instance for each haptor, call the
 *       addHaptor() with the single haptor to attach and then
 *       you can startPattern() on all the patterns together.
 *
 *
 * @param[out] pattern This is the pointer to a `pattern_desc *`
 * 					   returned by a precedent hl_initPattern()
 * 					   call. This `pattern_desc` should not be
 * 					   already running. If you want many instances
 * 					   of the same pattern, you need to follow all
 * 					   the steps to initialize it.
 *
 * @return uint8_t  The result is returned to indicate success
 * 					or error.
 *
 * @retval 0 Success
 *
 * @retval 1 Error
 *
 */
uint8_t hl_startPattern(pattern_desc * pattern)
{
	//Input Validation
	if( ( pattern == NULL ) || \
		( Null >= pattern->name ) || ( pattern->name >= Num_Patterns_Available))
	{
#ifdef HL_DEBUG
		send_string("hl_startPattern(): This pattern is not valid or not initialized.\r\n\0");
#endif
		return 1;
	}

	if( patternMap[pattern->name](pattern) )
	{
#ifdef HL_DEBUG
		send_string("hl_startPattern(): Problem in pattern Initiator.\r\n\0");
#endif
		return 1;
	}

	return 0;
}

/**
 * @brief **API exposed to the user** to force pattern execution stop.
 *
 * Normally, a well designed pattern will finish its job and then
 * free its instances (the pattern itself and all the haptors used)
 * to be eventually configured later.
 *
 * There are cases however, when you cannot wait for the pattern to
 * stop.\n
 * For example, imagine a continuous pattern that will run until the
 * user feel the feedback, in this case, the pattern must be stopped
 * by the application code (the condition cannot be coded inside the
 * pattern) calling hl_stopPattern() with the right pattern
 * descriptor.
 *
 * @param[out] pattern This is the pattern descriptor reference
 * 					   of a running pattern.
 *
 * @return uint8_t  The result is returned to indicate success
 * 					or error.
 *
 * @retval 0 Success
 *
 * @retval 1 Error
 */
uint8_t hl_stopPattern(pattern_desc *pattern)
{
	haptor_desc *tmp = NULL;
	haptor_desc *tmp2 = NULL;

	// Input validation
	if( (pattern == NULL) || \
			( Null >= pattern->name ) || ( pattern->name >= Num_Patterns_Available))
	{
#ifdef HL_DEBUG
		send_string("hl_stopPattern(): This pattern is not valid or not initialized.\r\n\0");
#endif
		return 1;
	}

	if( pattern->continuator == NULL )
	{
#ifdef HL_DEBUG
		send_string("hl_stopPattern(): This pattern is not active.\r\n\0");
#endif
		return 1;
	}

	if( pattern->activeHaptorList == NULL )
	{
#ifdef HL_DEBUG
		send_string("hl_stopPattern(): This pattern has no haptor linked to it (STRANGE).\r\n\0");
#endif
		return 1;
	}

	tmp  = pattern->activeHaptorList;
	tmp2 = NULL;
	do
	{
		TIM_Channel_DutyChanger( 0, tmp->id );

		tmp2 = tmp->nextHaptor;

		tmp->nextHaptor = NULL;
		tmp->activePattern = NULL;

		tmp = tmp2;
	}
	while( tmp );

	pattern->name             = Null;
	pattern->userParams       = NULL;
	pattern->continuator      = NULL;
	pattern->activeHaptorList = NULL;

	return 0;
}
