/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file   hapticLib.h
 *
 * @brief  **User API Module** header
 *
 * This file is part of the **User API Module** of HapticLib.
 *
 * This is the only file to be included by a project to use
 * HapticLib.
 *
 * This file is the entry point for the HapticLib user.
 *
 * All the High Level API the user _should_ use, are listed here.
 *
 * @note Please make sure the right symbols are set when invoking
 *       the compiler. (check the Makefile to be sure).
 *
 * @note For example, using the *STM32VLDISCOVERY* platform these
 *       symbols are needed:
 * 		  - -DSTM32VLDISCOVERY (required by HapticLib **Platform
 * 		     Specific Module**)
 * 		  - -DSTM32F10X_MD_VL (required by STM StdPeriph)
 * 		  - -DUSE_STDPERIPH_DRIVER (required by STM StdPeriph)
 *
 * @author Leonardo Guardati
 * @version 0.7
 * @date 2012
 *
 */

/**
 * @mainpage Main Page
 *
 * [TOC]
 *
 * # Introduction {#intro}
 *
 * Welcome to the HapticLib documentation repository.
 *
 * HapticLib is a haptic library for embedded systems based on
 * microcontrollers  like  ARM,   MSP430,   AVR,   PIC.
 *
 * Haptic feedback is obtained driving multiple vibro tactile
 * devices connected to the embedded system.
 *
 * All the documentation about HapticLib can be found here; some
 * of  it  is auto generated from source code, so it will always
 * reflect     the     latest    version    of     the     code.
 *
 *
 * [Download](HapticLib.tar.gz) the latest version of HapticLib.\n
 * [Download](HapticLib.pdf) the PDF version of the documentation.\n
 * [Download](workspace_keil.zip) the KEIL uVision4 workspace package
 * updated to the last release version.
 *
 * # News {#news}
 *
 * The main features added to the last version of HapticLib are:
 *
 *    - **Multi-Haptor** capability! Now HapticLib can manage
 *      multiple haptors simultaneously!!!
 *    - Formalized a new Template for the pattern generators.
 *    - Big changes everywhere to reflect new multi-haptor
 *      architecture.
 *    - Started **Platform Specific Module** documentation section
 *      under Architecture page.
 *
 * See the [Changelog](#changelog) for a detailed list of changes
 * from last version.
 *
 *
 * # Architecture Overview
 *
 * How HapticLib is structured and how all the pieces work together.
 *
 * The library is developed with modularity in mind. The functional
 * domains  are  decoupled  and  kept  separate  to  allow for easy
 * extensibility.
 *
 * The main modules are:
 *    - **User API**
 *    - **Platform Specific**
 *    - **Pattern Generators**
 *
 *
 * To know more:
 *    - Go to [Architecture](@ref hl_architecture).
 *
 * # For Users of HapticLib  {#users}
 *
 * What to setup to develop an application
 * using HapticLib in different environments.
 *
 * Specific instructions are provided for **Linux** and **Windows** OS.
 *
 * Setting up the environment to work with HapticLib can be hard, that's
 * why a detailed guide has been created to help the user get up and
 * running through the components needed and start coding his
 * application as soon as possible.
 *
 *    + Go to [User Guide](@ref user_guide).
 *    + Go directly to [Linux Instructions](@ref lin_info).
 *    + Go directly to [Windows Instructions](@ref win_info).
 *
 *
 * # For  Developers of HapticLib {#developers}
 *
 * The internal details of HapticLib.
 *
 * This page is useful for developers who want to modify or extend
 * the HapticLib library.
 *
 * Detailed description on some techniques used inside HapticLib is
 * given.
 *
 * For example, the **Pattern Generator Template** is a useful tool to use to
 * easily integrate new patterns inside the library.
 *
 * HapticLib also offer **Debugging Features**; it is a set of tools very handy, but
 * attention must be paid to avoid problems.
 *
 * For details,
 *    - Go to [Developer Guide](@ref dev_guide).
 *
 * # Changelog {#changelog}
 *
 * List of changes between versions of HapticLib.
 *
 *  ## v0.7 {#v07}
 *  - Added support to KEIL uVision4 IDE. Now development can be done using
 *    this IDE (and its toolchain)
 *  - Minor fix to library code to remove errors when compiling with armcc.
 *  - Fixed problem on HapticLevel Demo: changed SPI interface to I2C.
 *
 *
 *  ## v0.6 {#v06}
 *  - Added HapticLevel Demo (need to be tested with actual interface)
 *  - Completed documentation on Architecture page.
 *  - Completed documentation on Developer Guide page.
 *
 *
 *  ## v0.5 {#v05}
 *  - Implemented new Multi-Haptor architecture.
 *  - STM32VLDISCOVERY now supports MAX_HAPTORS=4, and will raise in
 *    the next releases.
 *  - Reorganized almost data structures to reflect new architecture.
 *  - Formalized a new pattern generator Template ( initiator/continuator).
 *  - Added data structures to increase code readability and compiler
 *    check enforcing (to work around void* argument passing).
 *  - Fixed send_int() to skip leading zeros from printing.
 *  - Fixed send_int() bug preventing 0x0 to be printed at all.
 *  - Moved the Pattern Scheduler in the **User API** module.
 *  - Updated all the demos to make use of the new Multi-Haptor capability.
 *  - Cleaned STM32VLDISCOVERY Platform code. Now The Timers are used in
 *    a cleaner way.
 *  - Moved the haptors GPIO to stop interfering with the User LED
 *    and Debug LED.
 *  - Fixed a bug in the Makefile that prevented Release version to
 *    fully work.
 *  - Fixed a bug in the Makefile that skipped updated .bin target from
 *    being built.
 *  - Started **Platform Specific Module** documentation section
 *    under Architecture page.
 *  - Restructured directory layout for patterns: now each pattern has
 *    its own sub-directory.
 *  - Restructured directory layout for platform: now each platform has
 *    its own sub-directory.
 *  - Updated demos Makefile to the new directory structure.
 *  - added utility function to convert duty cycles honoring min_duty
 *    and max_duty.
 *  - Almost completed Architecture documentation page.
 *
 *
 *  ## v0.4 {#v04}
 *  - Created CVS Repository to track code versioning and team development.
 *  - Moved SysTick configuration from hapticlib.c to platform specific code.
 *  - Moved debugging support code from hapticlib.c to hl_Debug.{h,c}.
 *  - Single-Haptor working implementation of the haptic "Impact Pattern".
 *  - Detailed documentation on "Impact Pattern" with theoretical references.
 *  - "Impact Pattern" samples results from Matlab script provided.
 *  - Standardized "Pattern Generators" implementation and documentation.
 *  - Created a Pattern Generator Template to use as starting point to 
 *  		create new patterns.
 *  - Cleaned the pattern header file inclusion system to flawlessly 
 *  		export pattern specific symbols to user program.
 *  - Added preliminary implementation for multi-haptor support. Now 
 *  		the API has changed, but the Platform Specific code still 
 *  		doesn't use multi haptors.
 *  - Cleaned Debugging features inclusion system.
 *  - Updated and expanded code documentation to reflect all code changes.
 *  - New structure for Library documentation.
 *  - Expanded Library documentation.
 *  - Increased granularity for PWM duty-cycle. Now there are 65535 values. 
 *  - bug fixes.
 *
 *
 *  ## v0.3 {#v03}
 *  - Makefile now eclipse friendly.
 *  - Makefile now works fine with Windows (needs basic unix commands [cygwin]).
 *  - Other Makefile bug fixes.
 *  - Elegant modularization of pattern generation functions using function pointers.
 *  - Created directory structure to insert new patterns.
 *  - Code documentation now is in sync with code.
 *
 *  ## v0.2 {#v02}
 *  - Cleaned the Makefile structure.
 *  - Added Demos/Project_Template skeleton for new applications.
 *  - Moved SysTick configuration inside hapticLib.c.
 *  - Fixed bug on conditional code pre-processing with HL_DEBUG symbol.
 *
 *  ## v0.1 {#v01}
 *  - First version of the library.
 *
 *
 *
 * # Additional Info {#additional_info}
 *
 *  ## Copyright Notice and License Terms and Conditions
 *
 *  Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>\n
 *  Copyright (c) 2012, Silvio Vallorani <silvio.vallorani@studio.unibo.it>
 *
 *  Permission  to  use,  copy,  modify,  and/or distribute this
 *  software  for  any  purpose  with  or  without fee is hereby
 *  granted,  provided  that the above copyright notice and this
 *  permission notice appear in all copies.
 *
 *  THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 *  ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 *  IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 *  EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 *  INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 *  WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 *  WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 *  TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 *  USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *    @version	0.7
 *    @authors	Leonardo Guardati
 *    @authors	Silvio Vallorani
 *    @date	2012
 *
 */

/**
 * @page hl_architecture Architecture
 *
 * [TOC]
 *
 * # HapticLib Structure Overview {#structure_overview}
 *
 *  HapticLib structure is composed of different parts working
 *  together to expose a simple API to the library user.
 *
 *  The main blocks are:
 *  	- **User API** module.
 *  	- **Platform Specific** module.
 *  	- **Pattern Generator** module.
 *
 *
 *  **Concept of Operations:**
 *
 * The user program code uses documented API from the *User API* module
 * (basically \#`include "hapticLib.h"` lists the API function offered).
 * The *User API* module then forward the application requests to the
 * other modules.
 * For the initialization phase, and for any hardware interaction,
 * the *Platform Specific* module is always called ultimately.
 * For anything related to the Haptic patterns generation, the
 * *Pattern Generator* module is the one doing the hard work. Here the
 * *User API* module act just as proxy for the application code to
 * call the right pattern generator function, allowing for transparent
 * extension of the Library without breaking existing code.
 *
 *  ![HapticLib Architecture](@ref HapticLibArchitecture.png)
 *  \image latex HapticLibArchitecture.png HapticLib Architecture
 *
 * ## User API Module: The high-level side {#high_level}
 *
 *   The **User API** module is the direct interface to the HapticLib
 *   user's application code.
 *   The developer should start from this module to learn how to use
 *   HapticLib.
 *
 *   Here is a simple use case.
 *
 *   The program starts initializing the system.
 *
 *   		uint32_t frequency    = 12000000;  // PWM frequency set to 12MHz.
 *   		uint8_t  sampleDelay  = 10;	       // PWM inter-samples delay 10ms.
 *   		uint8_t  numHaptors   = 3;         // Number of haptors used
 *
 *   		haptor_desc *myHaptors;			   // haptor descriptors array.
 *
 *   		myHaptors = hl_configure(frequency, sampleDelay, numHaptors);
 *
 * @note The number of haptors available depend on the hardware platform
 *       used and on the *Platform Specific* module support for that platform.
 *
 *    Then the program must initialize the patterns used in the application:
 *
 *    		pattern_desc *myPattern;
 *
 *    		myPattern = hl_initPattern(Test, NULL);
 *
 *    		hl_addHaptor(&myHaptors[1],myPattern);
 *
 *    Now the pattern can be started:
 *
 *    		hl_startPattern(myPattern);
 *
 *    And stopped (if the pattern has not finished yet):
 *
 *			Delay(2000);
 *
 *    		hl_stopPattern(myPattern);
 *
 * @note Make sure to read the documentation of the pattern to know what
 *   data type is expected, and what are the correct value ranges
 *   for every parameter.
 *
 * That's it!
 *
 * From an Interface point of view, there is nothing more to do for the
 * system to work. It is up to the developer now to implement the right behavior of the
 * application.
 *
 * HapticLib offers a lot more than this simple and plain example.
 * You can simultaneously start different patterns on different
 * haptors. The patterns will run concurrently together.\n
 * Even more, multi-haptors patterns can be setup to drive different
 * haptors under the same haptic logic (just call `hl_addHaptor()`
 * multiple times on the same pattern).\n
 * HapticLib uses a pattern scheduler to implement this features. If
 * you want to know more, pleas refer to the
 * [Developer Guide](@ref dev_guide) page.
 *
 * Please, make sure to also read all the other documentation; in
 * particular check:
 *  - The hardware details of the platform used.
 *  - The details of the pattern generated; the theory behind the haptic
 *  	feedback and the meaning of the parameters required.
 *
 * @see Please refer to the
 * 		[Pattern Generators module](@ref pattern_generators) to know
 * 		what patterns are available and learn all the details on
 * 		them, like the name to use, and any optional parameter
 * 		required to work.
 *
 * ## Platform Specific Module: the low-level side {#low_level}
 *
 *   The interaction with the hardware is done in this module.
 *   Actually this module is composed of a pair of .c/.h files
 *   for each hardware platform supported by the library.
 *
 *   During the initial configuration phase, the high-level
 *   hl_configure() function calls the right implementation
 *   of the low level routines based on the platform specific
 *   symbols defined. (for example
 *   [STM32VLDISCOVERY](@ref STM32VLDISCOVERY)).
 *
 *   If a platform provide a peripheral driver library, this module
 *   can make use of it greatly simplifying the code.
 *   For example the STM32VLDISCOVERY board is based on a STM32F10x
 *   MCU that is supported by STM's StdPeriph Library.
 *
 *   Every platform have its characteristics, so it is important to
 *   read about HapticLib implementation for that platform that impact
 *   on the high level behavior of the application.
 *
 *   Here is the list of supported platforms:
 *
 *   ### STM32VL-DISCOVERY {#stm32vldiscovery}
 *
 *     This board from STM uses a STM32F100RB ARM based MCU. The
 *     library development started on this board, so it is well
 *     supported.
 *
 *     ![STM32VLDISCOVERY](STM32VLDISCOVERY.png)
 *     \image latex STM32VLDISCOVERY.png STM32VLDISCOVERY
 *
 *     Here the interesting specifications of the MCU from a
 *     HapticLib perspective:
 *     		- up to 24MHz Core and Peripheral bus speed.
 *   		- up to 7 TIMers  (resulting in more than 20 PWM
 *   					channels directly mappable to GPIOs).
 *   		- additional System Timer (SysTick) to measure
 *   					time delays.
 *
 *   	For [detailed informations][mcu_specs] refer to STM
 *   	documentation.
 *
 *   	The STM32VL-DISCOVERY Developer Board features:
 *   			- a lot of IO expansion capabilities, useful
 *   					for multi-haptor scenarios.
 *   			- a built-in user button for basic interaction.
 *   			- two built-in LEDs for basic application feedback
 *   			 	and debug.
 *
 *		Also from STM, the [StdPeriph] is used to simplify hardware
 *		addressing.
 *
 *		@note HapticLib doesn't distribute the STM's StdPeriph
 *			Standard Peripheral. But it is possible to
 *			[download][StdPeriph] the package directly from ST.
 *			Once downloaded just uncompress the content inside
 *			the HapticLib root directory and all the features will
 *			be available.
 *
 *      #### Haptic Feedback layout
 *
 *      The STM32VL-DISCOVERY Dev Board, can generate more than
 *      20 PWM control signals directly mappable to GPIO via
 *      Alternate Functions. However the HapticLib's
 *      STM32VLDISCOVERY Platform Specific module only supports
 *      4 haptic devices to be driven by the application.
 *      Future release may add support for additional haptic
 *      devices for this platform.
 *
 *      Here is the haptic device hardware layout supported:
 *
 *      ![STM32VLDISCOVERY_devs](STM32VLDISCOVERY_devs.png)
 *      \image latex STM32VLDISCOVERY_devs.png STM32VLDISCOVERY Devices
 *
 *      The Serial link available on the PB I/O port, is enabled
 *      only when the [HL_DEBUG](@ref HL_DEBUG) symbol is defined at
 *      compile time.
 *
 * ## Pattern Generators Block: the haptic feedbacks {#pattern_generators}
 *
 *   To understand how the **Pattern Generators** module works, let's
 *   start describing the sequence of events happening when the user
 *   starts a pattern.
 *
 *   When the user application code calls hl_startPattern() (after the
 *   initialization step), the **User API** module forward the call
 *   to the initiator of the pattern passed as parameter.\n
 *   Once called, the pattern initiator validate the user parameters.
 *   If the parameters are valid, the initiator will setup the its
 *   initial status, and the pattern instance becomes running.\n
 *   When the pattern instance becomes running, the **pattern
 *   scheduler** will start calling its continuator every time the
 *   inter sample delay elapses.\n
 *   When the pattern ends (if it ever ends) the continuator will
 *   remove the pattern instance from the scheduler freeing the
 *   resources used (the haptors).\n
 *   Alternatively, if the pattern is still running and the user
 *   needs it to stop (or needs the haptors for other patterns)
 *   the hl_stopPattern() API function can be called to force the
 *   pattern de-scheduling and resources freeing.
 *
 *   Now that the running flow of events has been described, we can
 *   show the Pattern Generator typical structure.
 *
 *   A Pattern Generator is a sort of module plugged in HapticLib.
 *   It is composed of two functions:
 *   	- the pattern **initiator**
 *   	- the pattern **continuator**
 *
 *   and two data structure type definitions:
 *      - the pattern **user parameters**
 *      - the pattern **status parameters**
 *
 *   The **initiator** for a pattern instance gets called only once;
 *   when the user application code calls hl_startPattern().\n
 *   Its job is to setup the initial status of the pattern and
 *   schedule the instance for running to the patternScheduler().
 *
 *   The **continuator** for a pattern instance is called by the
 *   patternScheduler() every inter-sample delay (after the
 *   initiator scheduled it) as long as the pattern instance is
 *   *running*. The haptic feedback logic of the pattern decides
 *   when to stop the instance. During its last run, the continuator
 *   will de-schedule the pattern instance from the scheduler, and
 *   free all the resources used (the haptors).
 *
 *   @note It is important to understand how a pattern works in order
 *         to know if the instance will stop by itself or if the user
 *         have to call hl_stopPattern() to put an end to it.
 *
 *   A Pattern Generator also defines two data structures to hold the
 *   informations needed at runtime.
 *
 *   The **user parameters** is a type definition based on a
 *   structure holding all the data the user can fine-tune when
 *   creating a new *pattern instance*.\n
 *   It is important to read the documentation of the pattern to be
 *   used, to understand the meaning and the use the pattern will
 *   make of every single parameter passed by th user.
 *
 *   To better understand, here is an example of user parameters for
 *   an existing pattern, the **impact pattern** generator.\n
 *   The user parameters accepted to personalize a pattern instance
 *   are:
 *   	- the impact **material**
 *   	- the impact **velocity**
 *
 *   One instance could send the haptic feedback of a **fast** impact
 *   on a **wood** surface, then another instance could send the
 *   feedback of a **slow** impact on a **rubber** surface.
 *
 *   Refer to the [impact pattern generator](@ref impact.c)
 *   documentation for detailed informations.
 *
 *   The **status parameters** structure is an internal container of
 *   the pattern instance status and progress. It is not exposed to
 *   the user, nevertheless it is important to understand that
 *   internally the pattern initiator will setup the starting status
 *   (probably depending on the provided  *user parameters*) and
 *   that the continuator will update the status parameters at every
 *   patternSchedule() call based on the logic and the progress of
 *   the pattern.
 *
 *   @note In a typical application, the *user parameters* are
 *   instantiated in the user application code (for example in the
 *   main.c module), while the *status parameters* are hold inside
 *   an internal indexing data structure.\n
 *   Typically the **user parameters** variable is set up before
 *   initializing the pattern because the initiator uses it to
 *   set the initial values for the *status parameters*.\n
 *   HapticLib architecture allow also an additional use case for the
 *   *user parameters*, that is a way to modulate the pattern
 *   *continuator* behavior during the pattern execution.
 *   The user has full control over the *user parameter* variable,
 *   and the *continuator* can poll the actual value of any user
 *   parameter during its executions.
 *
 * # Typical Use Scenarios {#use_scenario}
 *
 *  The following examples will describe some uses of the library
 *  showing the way to use the *User API* and the different kind
 *  of pattern existent.
 *
 *  Under the `Demos/` HapticLib sub-directory are present
 *  different demo applications.
 *
 *  ## Example 1: HapticWorld {#HapticWorld}
 *
 *  HapticWorld is the simplest demo possible, showing how to use
 *  HapticLib API.\n
 *  This demo activate a single haptor test pattern and then exits.
 *
 *  The main.c code is listed below, interleaved with the
 *  descriptions.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * #include "hapticLib.h"
 *
 * ~~~~~~~~~~~~~~~~
 *
 * This include will enable HapticLib in the application.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * enum demoHaptors {
 * 	Head          = 0,
 * 	Lhand         = 1,
 * 	Rhand         = 2,
 * 	Back          = 3,
 *
 * 	NumDemoHaptor = 4
 * };
 *
 * ~~~~~~~~~~~~~~~~
 *
 * This enumeration will make the code cleaner when referring to
 * the haptor descriptors array.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * int main(void)
 * {
 *
 * 	// Pointer to the array containing the setup haptors.
 * 	haptor_desc *myHaptors;
 *
 * 	// Initialize the Haptic System and get the haptors array.
 * 	myHaptors = hl_configure(24000000, 10, NumDemoHaptor);
 *
 * ~~~~~~~~~~~~~~~~
 *
 *  Here the reference to the set of haptors used by the application
 *  is declared and initialized. `NumDemoHaptor` can be less than the
 *  total available system haptors. From now on every haptor can be
 *  referenced in this way:
 *
 *  `myHaptor[Head]` for the first haptor
 *  `myHaptor[Lhand]` for the second, etc.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * 	// We are ready to send patterns.
 *
 * 	pattern_desc *myPattern;
 *
 * 	// Use a Test Pattern
 * 	myPattern = hl_initPattern(Test, NULL);
 *
 * ~~~~~~~~~~~~~~~~
 *
 * `myPattern` is the descriptor of the test pattern we want to use.
 * In this case the second argument of hl_initPattern() is `NULL`
 * because the Test Pattern does not accept any user parameter.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * 	// We need to tell the Test pattern which haptor to use
 * 	hl_addHaptor( &myHaptors[Head], myPattern);
 *
 * ~~~~~~~~~~~~~~~~
 *
 * The pattern is now setup, but it doesn't know yet on which haptor
 * to work. hl_addHaptor() link the given haptor to the given
 * pattern. In this case `myHaptor[Head]` is used.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * 	// Start Shaking
 * 	hl_startPattern(myPattern);
 *
 * ~~~~~~~~~~~~~~~~
 *
 * The hl_startPattern() call activates the feedback  execution.
 *
 * In this particular Pattern and with the settings given to
 * hl_configure()the pattern activation will last for some
 * time (about 2 seconds).
 *
 * ~~~~~~~~~~~~~~~~
 *
 * 	// Let it shake for a while...
 * 	Delay(1000);
 *
 * 	// ...or force its stop if we need the haptor for something else.
 * 	hl_stopPattern(myPattern);
 *
 * ~~~~~~~~~~~~~~~~
 *
 * To show the forced stopping of the pattern, hl_stopPattern() is
 * called after 1 second wait (in the middle of pattern execution).
 *
 * ~~~~~~~~~~~~~~~~
 *
 * #ifdef HL_DEBUG
 * 		send_string("\r\nHapticWorld: Going to sleep...goodbye!\r\n\0");
 * #endif
 *
 * 	// go to sleep
 * 	while(1) { __WFE(); }
 * }
 *
 * ~~~~~~~~~~~~~~~~
 *
 * In the end the program goes to sleep.
 *
 *
 *  ## Example 2: HapticCalibrator {#HapticCalibrator}
 *
 *  HapticCalbirator is a demo application that let the user
 *  calibrate the minimum PWM duty cycle for an haptor that
 *  delivers a vibration strong enough to be felt.
 *
 *  The HapticCalibrator demo will show how a continuous pattern
 *  works, how is possible to modify a user parameter during a
 *  pattern execution.
 *
 *  The main.c code is listed below, interleaved with the
 *  descriptions.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * #include "stm32f10x.h"
 * #include "hapticLib.h"
 *
 * ~~~~~~~~~~~~~~~~
 *
 * In addition to HapticLib include, this application will use
 * some hardware peripheral directly, that's the reason for the
 * STM include.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * int main(void)
 * {
 *
 * 	haptor_desc *myHaptor;
 * 	user_param   myParams;
 *
 * 	myHaptor = hl_configure(24000000,10,1);
 *
 * ~~~~~~~~~~~~~~~~
 * Here the reference to the haptor used and the pattern's user
 * parameters are defined.
 *
 * Then the haptor reference is initialized with hl_configure().
 *
 * ~~~~~~~~~~~~~~~~
 *
 * 	// User Button Event Configuration
 * 	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource0);
 * 	GPIO_EventOutputConfig(GPIO_PortSourceGPIOA, GPIO_PinSource0);
 *
 * 	EXTI_InitTypeDef InitStruct;
 * 	InitStruct.EXTI_Line    = EXTI_Line0;
 * 	InitStruct.EXTI_Mode    = EXTI_Mode_Interrupt;
 * 	InitStruct.EXTI_Trigger = EXTI_Trigger_Rising;
 * 	InitStruct.EXTI_LineCmd = ENABLE;
 *
 * 	EXTI_Init(&InitStruct);
 *
 * #ifdef HL_DEBUG
 * 	send_string("\r\n\r\n\0");
 * 	send_string("\t\tWelcome to HapticCalibrator!!\r\n\r\n\0");
 * 	send_string("Now we will calibrate the System's Haptors...\r\n\0");
 * 	send_string("As soon as you CLEARLY feel the vibration, hit the User Button!!!\0");
 * 	send_string(" ( Be quick! ;) )\r\n\r\n\r\n\0");
 * #endif
 *
 * 	Delay(1500);
 *
 * #ifdef HL_DEBUG
 * 	send_string("Wear the Haptor and when you feel ready, press the User Button.\r\n\0");
 * #endif
 *
 * 	while ( EXTI_GetFlagStatus(EXTI_Line0) != SET ) { __WFI(); } ;
 *
 * 	EXTI_ClearFlag(EXTI_Line0);
 *
 * ~~~~~~~~~~~~~~~~
 *
 * This piece of code is specific to this application. The EXTernal
 * Interrupt is configured with the User Button (PA0), and then the
 * program waits (sleeping) for the user to press the User Button
 * and let the calibration begin.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * 	myParams.constant.constant = 0x0000;
 * 	hl_startPattern( hl_addHaptor(myHaptor, hl_initPattern(Constant, &myParams) ) );
 *
 * ~~~~~~~~~~~~~~~~
 *
 * The pattern used in this application is the **Constant Pattern**.
 * It takes a user parameter `constant` that represent the duty
 * cycle of this haptor's PWM.
 *
 * The starting of the pattern here is condensed using call nesting
 * of functions.\n
 * To call hl_startPattern(), hl_addHaptor() must be called first,
 * and likewise hl_initPattern() is required by hl_addHaptor().
 *
 * So the actual sequence is:
 * 		- hl_initPattern()
 * 		- hl_addHaptor()
 * 		- hl_startPattern()
 *
 * 	The initial duty cycle is 0x0000.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * 	while ( myParams.constant.constant < 0xff00 )
 * 	{
 * #ifdef HL_DEBUG
 * 		send_string("Duty Cycle: \0");
 * 		send_int(myParams.constant.constant);
 * 		send_string("\r\n\0");
 * #endif
 * 		myParams.constant.constant += 10;
 *
 * 		Delay(1);
 *
 * 		if( EXTI_GetFlagStatus(EXTI_Line0) == SET )
 * 		{
 * 			EXTI_ClearFlag(EXTI_Line0);
 * #ifdef HL_DEBUG
 * 			send_string("GOOD! We found the minimum dutyCycle!\r\n\0");
 * #endif
 * 			GPIO_WriteBit(GPIOC, GPIO_Pin_8, Bit_SET);
 * 			break;
 * 		}
 * 	}
 *
 * ~~~~~~~~~~~~~~~~
 *
 * Now a cycle is started to increase the duty cycle at every
 * iteration. And when the user presses the User Button, the cycle is
 * interrupted.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * 	if( myParams.constant.constant >= 0xff00 )
 * 	{
 * #ifdef HL_DEBUG
 * 		send_string("BAD!! We reached almost the 100\% DutyCycle and you didn't press.\r\n\0");
 * 		send_string("Make sure your Haptor isn't broken!\r\n\0");
 * #endif
 * 	}
 * 	else
 * 	{
 * #ifdef HL_DEBUG
 * 		send_string("Now setting the minimum activating DutyCycle to: \0");
 * 		send_int(myParams.constant.constant);
 * 		send_string("\r\n\0");
 * #endif
 *
 * 		myHaptor->min_duty = myParams.constant.constant;
 * 	}
 *
 * ~~~~~~~~~~~~~~~~
 *
 * Here a check is made to verify the user really pressed the button
 * and the duty didn't reach the top instead.
 *
 * If the user pressed the button, the haptor's minimum duty cycle is
 * updated with the value set by the user.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * 	hl_stopPattern(myHaptor->activePattern);
 *
 * ~~~~~~~~~~~~~~~~
 *
 * The **Constant Pattern** is a non stop pattern, this means
 * the user application must call hl_stopPattern() to end the
 * pattern and free the haptor.
 *
 * ~~~~~~~~~~~~~~~~
 *
 * #ifdef HL_DEBUG
 * 	send_string("\r\nGoodbye....going to sleep...\r\n\0");
 * #endif
 *
 * 	while(1) { __WFE(); }
 * }
 *
 * ~~~~~~~~~~~~~~~~
 *
 * In the end the program goes to sleep.
 *
 *  ## Example 3: HapticLevel {#HapticLevel}
 *
 *  HapticLevel is a very simple, real world example
 *  demo showing how easy is to implement haptic
 *  feedback embedded applications.
 *  HapticLevel acts as an electronic Level to measure
 *  a surface's tilt.
 *  The device uses 4 haptors (one for each direction)
 *  and one 3-axis accelerometer (STM's LIS302DL).
 *  The haptors are connected to the board's GPIO, the
 *  sensor is connected to the I2C1 peripheral, and the
 *  serial console (enabled only in debug versions) is
 *  connected to USART3 as usual.
 *
 *  Here is the complete system layout:
 *
 *  ![HapticLevel](@ref HapticLevel.png)
 *  \image latex HapticLevel.png HapticLevel Layout
 *
 *  The haptors are placed in pairs along orthogonal
 *  axes.
 *  When the device is placed on a surface, it will
 *  "display" the tilt activating one haptor for each
 *  direction.
 *
 *  Now that the device has been described, let's look
 *  at HapticLevel's code.
 *
 *  HapticLib knows nothing about the LIS302DL sensor
 *  and all the relevant code is jut part of the demo.
 *
 *  To unclutter main.{c,h} source code files, another
 *  module has been used to place all sensor specific
 *  code.
 *
 *  LIS302DL support SPI and I2C interface. I2C has been
 *  used.
 *  Routines to manage communications with the sensor
 *  are based on existing code by Michele Magno and
 *  Bojan Milosevic (MicreLab University of Bologna)
 *  located on lis302dl.c, lis302dl.h HAL_LIS3LV02DL.h
 *  source code files.
 *
 *  In main.c the fist thing the program does is to
 *  init the I2C1 peripheral and then init the sensor
 *  as well:
 *
 *  ~~~~~~~~~~~~~~~~~~~~~~~~
 *  // I2C1 peripheral Initialization
 *  LIS3LV02DL_I2C_Init();
 *
 *  // LIS302DL Sensor Initialization
 *  LIS3LV02DL_Acc_Init();
 *  ~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  The next thing main() does is to initialize HapticLib
 *  library, configuring the four haptors, setting up
 *  the Constant pattern, and attaching the haptors to the
 *  pattern.
 *
 *  ~~~~~~~~~~~~~~~~~~~~~~~~
 *  // Haptors Initialization
 *  // Initialize the Haptic System (setting the PWM global period in Hz)
 *  haptors = hl_configure(24000000,10, NumLevelHaptors);
 *  // Start each haptor
 *  for( i=0 ; i<NumLevelHaptors ; i++)
 *  {
 *    levelParams[i].constant.constant = 0;
 *      if(  hl_startPattern(
 *             hl_addHaptor(  &haptors[i],
 *                    hl_initPattern(Constant,&levelParams[i]) ) )  )
 *      {
 * #ifdef HL_DEBUG
 *      send_string("HapticLevel: Problem Starting Haptor \0");
 *      send_int(i);
 *      send_string(".\r\n\0");
 * #endif
 *      }
 *  }
 *  ~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *
 *  The starting intensity is set to 0 for each pattern, so they will
 *  not vibrate.
 *  Now the actual application logic kicks in; the program enters an
 *  endless loop where the values of x,y,z acceleration are
 *  read from the sensor.
 *  The z-axis is used to sense the actual orientation of the device
 *  (upside or downside).
 *  Then the x and y axis are used to decide which haptor of each pair
 *  will vibrate with a new constant value proportional to the entity
 *  of directional tilt. The other haptor of the pair is simply kept to
 *  a constant value of 0 to prevent vibration.
 *
 *  ~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  while(1) {
 *
 *  // Get X-axis value
 *  LIS3LV02DL_Acc_Read_RawData(&tmp,LIS_A_OUT_X_L_ADDR);
 *  x = (int8_t) (tmp >> 8);
 *
 *  // Get Y-axis value
 *  LIS3LV02DL_Acc_Read_RawData(&tmp,LIS_A_OUT_Y_L_ADDR);
 *  y = (int8_t) (tmp >> 8);
 *
 *  // Get Z-axis value
 *  LIS3LV02DL_Acc_Read_RawData(&tmp,LIS_A_OUT_Z_L_ADDR);
 *  z = (int8_t) (tmp >> 8);
 *
 *  #ifdef HL_DEBUG
 *      send_string("HapticLevel: X-Value \0");
 *      send_int(x);
 *      send_string(".\r\n\0");
 *  #endif
 *
 *  // positive orientation
 *  if( z >= 0 ) {
 *          if( x >= 0 ) {
 *              levelParams[Up].constant.constant     = 4096*x;
 *              levelParams[Down].constant.constant   = 0;
 *          }
 *          else {
 *              levelParams[Down].constant.constant   = -4096*x;
 *              levelParams[Up].constant.constant     = 0;
 *          }
 *          if( y >= 0 ) {
 *              levelParams[Left].constant.constant   = 4096*y;
 *              levelParams[Right].constant.constant  = 0;
 *          }
 *          else {
 *              levelParams[Right].constant.constant  = -4096*y;
 *              levelParams[Left].constant.constant   = 0;
 *          }
 *  }
 *  else { // negative orientation
 *          if( x <= 0 ) {
 *              levelParams[Up].constant.constant     = -4096*x;
 *              levelParams[Down].constant.constant   = 0;
 *          }
 *          else {
 *              levelParams[Down].constant.constant   = 4096*x;
 *              levelParams[Up].constant.constant     = 0;
 *          }
 *          if( y <= 0 ) {
 *              levelParams[Left].constant.constant   = -4096*y;
 *              levelParams[Right].constant.constant  = 0;
 *          }
 *          else {
 *              levelParams[Right].constant.constant  = 4096*y;
 *              levelParams[Left].constant.constant   = 0;
 *          }
 *      }
 *
 *      // Sleep
 *      Delay(100);
 *
 *  }
 *
 *  while(1) { __WFE(); }
 *  ~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  The main() code enters an endless loop where it
 *  polls for new values of the 3 vectors (x,y,z)
 *  from the sensor.
 *
 *  The single direction value is retrieved calling
 *  LIS3LV02DL_Acc_Read_RawData() with the first
 *  argument being a pointer to int16_t value and the
 *  second argument the direction to be retrieved.
 *
 *  The values are then casted to 8 bit signed values.
 *
 *  The first check is on z-axis value to understand
 *  if the device is up-face or down-face.
 *
 *  Then the x and y-axis are checked to know the
 *  device tilt on each direction.
 *
 *  If the axis value is positive, a scaled value of
 *  the sensed amplitude is set as duty-cycle for one
 *  haptor. (and a zero duty is set for the opposite
 *  one).
 *  If the axis value is negative, the same happen but
 *  with inverted haptors.
 *
 *  Note the negative scaling coefficient for negative
 *  direction. This is needed in order to have coherent
 *  proportional intensity variation on tilt.
 *
 *  The last line of code should never be reached.
 *
 * # Application Debugging Feature {#debug_feature}
 *
 *  HapticLib offers some debugging features to ease
 *  application development and bug hunting.
 *
 *  The debugging features are made of two utility
 *  functions:
 *
 *   - send_int()
 *   - send_string()
 *
 *  Implementing those functions, the user won't need
 *  any additional code library for basic I/O across
 *  a serial link.
 *
 *  All the debugging code inside the library gets
 *  conditionally compiled if the preprocessor symbol
 *  HL_DEBUG is defined by the user, allowing the
 *  complete removal of any debugging code on the
 *  release version of the application.
 *
 *  The user application code must adhere to this
 *  technique, enclosing any debug related code
 *  inside preprocessor rules:
 *
 *  ~~~~~~~~~~~~~~~~
 *
 *  #ifdef HL_DEBUG
 *
 *  	// Debugging code
 *
 *  #endif
 *
 *  ~~~~~~~~~~~~~~~~
 *
 *  The HL_DEBUG preprocessor symbol must be defined
 *  in order to enable the debugging features.
 *  Th best place to do this is adding the definition
 *  on the compiler invocation command. For example:
 *
 *  ~~~~~~~~~~~~~~~~
 *
 *    gcc ... -DHL_DEBUG ...
 *
 *  ~~~~~~~~~~~~~~~~
 *
 *  The Makefile available from the template Demo application
 *  defines a variable to easily manage the compilation of DEBUG
 *  and/or RELEASE target version of the application.
 *
 *   ## send_int()
 *
 *   This utility function allow the transmission of a string
 *   representation of any unsigned long integer number ( uint32_t ).
 *
 *   The number is always printed in hexadecimal format ( 0x... )
 *   and additionally, if the value is less than 100000, the
 *   decimal representation is print too.
 *
 *   For example:
 *
 *   ~~~~~~~~~~~~~~~
 *
 *    send_int(256);
 *
 *   ~~~~~~~~~~~~~~~
 *
 *   will output the following string:
 *
 *   ~~~~~~~~~~~~~~~
 *
 *    0x100 (256)
 *
 *   ~~~~~~~~~~~~~~~
 *
 *   meanwhile:
 *
 *   ~~~~~~~~~~~~~~~
 *
 *    send_int(456789);
 *
 *   ~~~~~~~~~~~~~~~
 *
 *   will output:
 *
 *   ~~~~~~~~~~~~~~~
 *
 *    0x6f855
 *
 *   ~~~~~~~~~~~~~~~
 *
 *   ## send_string()
 *
 *   This utility function allow the transmission of a string
 *   across the serial link.
 *
 *   The string will be trimmed to 255 characters.
 *
 *   If the string passed has a null terminator ( '\0' ), only
 *   the characters preceding it will be printed.
 *
 *   For example:
 *
 *   ~~~~~~~~~~~~~~~~
 *
 *     send_string("Hello World!\n\rGoodbye World!\0Not this");
 *
 *   ~~~~~~~~~~~~~~~~
 *
 *   will output:
 *
 *   ~~~~~~~~~~~~~~~~
 *
 *     Hello World!
 *     Goodbye World!
 *
 *   ~~~~~~~~~~~~~~~~
 *
 *   ## Platform specific implementations
 *
 *   Both send_int() and send_string() functions rely on the
 *   function send_char() to actually output every single
 *   character.
 *
 *   send_char() is a platform specific function.
 *
 *   Each platform must implement the send_char() function
 *   to enable the debugging features.
 *
 *   For example, the STM32VLDICOVERY platform module uses
 *   the USART3 as serial link for debugging messages.
 *
 *
 *   [mcu_specs]: http://www.st.com/internet/mcu/product/216844.jsp
 *   [StdPeriph]: http://www.st.com/internet/com/SOFTWARE_RESOURCES/SW_COMPONENT/FIRMWARE/stm32f10x_stdperiph_lib.zip
 *
 */

/**
 * @page user_guide User Guide
 *
 *
 * This page is directed to developers of haptic applications
 * on embedded platforms.
 *
 * Here is explained how to setup the development environment
 * in different configurations.
 *
 *
 * # Operating Systems supported
 *
 * Specific instruction on how to setup the system follows.
 *
 *   - @subpage lin_info
 *
 *     This page documents the tools needed to setup the
 *     environment on a Linux system.
 *
 *   - @subpage win_info
 *
 *     This page documents the tools needed to setup the
 *     environment on a Windows system.
 *
 */

/**
 * @page win_info Windows
 *
 * @note This page is directed to developers of haptic applications
 * on embedded platforms.
 *
 * Here is explained how to setup the development environment
 * in different configurations using **Windows** Operating System.
 *
 * [TOC]
 *
 * \n
 * # Preliminary Setup {#win_requirements}
 *
 * Before start using HapticLib you need to correctly prepare the
 * system. Haptic applications and the library (including this documentation)
 * are compiled using the provided Makefiles (refer to
 * [No IDE](@ref win_noide) paragraph for explanations).\n
 * Therefore a version of [GNU make.exe][win_make] for Windows is necessary.
 *
 * Furthermore some other Unix Tools are necessary to correctly execute
 * all the operations defined in Makefiles:
 * 	- bash.exe
 * 	- rm.exe
 * 	- mkdir.exe
 * 	- cp.exe
 * 	- mv.exe
 *
 * For these reasons a more complete Unix Tools Package for Windows is
 * suggested.\n
 * For example two tested packages are:
 * - [MinGW][win_mingw]
 * - [Cygwin][win_cygwin]
 *
 * @note For Cygwin: there is no need to install everything, select
 * 		only the basic, developer and library parts.
 *
 * After choosing the preferred package, make sure the above commands
 * are available in PATH.
 *
 * @note Developers interested in building documentation must have
 * 		in PATH Doxygen, LaTeX and ghost script tools. Please refer
 * 		to [Developer Guide](@ref dev_guide) for detailed informations.
 *
 * \n
 * # Tool-chains supported {#win_toolchains}
 *
 * Now that the operating system is correctly setup, a tool-chain is
 * needed to compile the code. Make sure the compiler collection binaries
 * (arm-none-eabi-*) are available in PATH.
 *
 * A list of tool-chains tested, divided by Platform kind, is provided.
 *
 * ## ARM&reg; {#win_arm}
 *
 * All the arm tool-chains are similar (all originates from gcc) and
 * probably will work just fine.\n
 * The tested ones are:
 *
 * ### Linaro - GCC ARM Embedded {#win_linaro}
 *
 * Get the [Linaro - GCC ARM Embedded][win_linaro] tool-chain.
 *
 * ### Atollic - TrueSTUDIO for ARM Lite {#win_atollic}
 *
 * Get the [Atollic - TrueSTUDIO for ARM Lite][win_atollic] IDE that
 * include tool-chain and [gdb-server](@ref win_gdbservers).
 *
 * @note The executable downloaded can be extracted like a zip archive
 * 		 and then only the tool-chain folder (PATH_WHERE_EXTRACT\\$_OUTDIR\\ARMTools)
 * 		 can be token and used as is.
 *
 * @warning Atollic tool-chain binaries are called arm-atollic-eabi-*
 * 			instead of arm-none-eabi-*.
 *
 * ### Mentor Graphics - Sourcery CodeBench Lite Edition {#win_codesourcery}
 *
 * Get the [Mentor Graphics - Sourcery CodeBench Lite Edition][win_codesourcery]
 * tool-chain.
 *
 * @note With this tool-chain comes *cs-make.exe* and *cs-rm.exe*
 * 		 that can be used if preferred.
 *
 * ### ARM - ARMCC toolchain
 *
 * This toolchain comes with KEIL's uVision IDE. It works and is supported in
 * HapticLib development.
 * You can [download here](https://www.guardati.it/hapticLib/workspace_keil.zip)
 * a ready to use uVision4 workspace with all it's needed.
 *
 * ## Texas Instruments&trade; MSP430&reg; {#win_msp430}
 *
 * At the moment the library dooesn't support this embedded platform.
 * Probably there should not be any problem porting HapticLib to it.
 *
 * ## Microchip&trade; PIC32&reg; {#win_pic32}
 *
 * At the moment the library dooesn't support this embedded platform.
 * Probably there should not be any problem porting HapticLib to it.
 *
 * ## Atmel&trade; AVR32&reg; {#win_avr}
 *
 * At the moment the library dooesn't support this embedded platform.
 * Probably there should not be any problem porting HapticLib to it.
 *
 * \n
 * # Flasher utilities {#win_flasher}
 *
 * ##ST Microelectronics&trade; STM32VLDISCOVERY
 *
 * For correctly attach and use this developing board to Windows based
 * systems, drivers and flasher utility are needed.\n
 * Get the official [ST Microelectronics - ST-LINK Utility][win_st_utility]
 * package and install them.
 *
 * \n
 * # GDB Servers supported {#win_gdbservers}
 *
 * ##ST Microelectronics&trade; STM32VLDISCOVERY
 *
 * Projects made for this dev-board can be debugged using gdb
 * that include two elements: gdb-server and gdb-client. While the
 * client is included in tool-chains (arm-none-eabi-gdb.exe), the
 * server must be installed apart.\n
 * A gdb-server for Windows tested with this dev-board is provided
 * by Atollic.\n
 * Get the [Atollic - TrueSTUDIO for ARM Lite][win_atollic], extract
 * the executable like a zip archive and take the folder
 * PATH_WHERE_EXTRACT\\Servers\\ST-LINK_gdbserver .
 * @warning ST-LINK_gdbserver.exe doesn't work out of the box: some
 * other tricks are needed:
 * 		- Override STLinkUSBDriver.dll with the one that is provided
 * 		by STM in [ST-LINK Utility](@ref win_flasher).
 * 		- Ignore the ST-Link_V2_USBdriver.exe included. Those
 * 		provided by STM in [ST-LINK Utility](@ref win_flasher) must
 * 		be used.
 * 		- ST-LINK_gdbserver.exe must be invoked with `-d` parameter
 * 		which enable SWD Debug mode.
 * 		- The default listening port of the gdb-server is 61234 but
 * 		can be changed invoking it with `-p port_num` parameter.
 *
 * \n
 * # IDE supported {#win_ides}
 *
 * ## No IDE {#win_noide}
 *
 * The library is versatile enough to be used without a complete
 * IDE; in fact you can compile the application and the library
 * (including this documentation) using the provided Makefiles.
 *
 * The first versions of HapticLib with the demo applications (up to v0.2)
 * were developed in linux using only the [vim](http://www.vim.org/)
 * text editor, but every other text editor can be used too.
 * The code was then compiled using
 * [GNU make](http://www.gnu.org/software/make/) which in turns
 * uses the gcc toolchain to produce the binary code.
 * In Windows refer to [pre-requirements](@ref win_requirements) and
 * [tool-chain](@ref win_arm) paragraphs. To flash the code on the
 * device, [texane/st-link](https://github.com/texane/stlink/) was
 * used. In Windows refer to [flasher utility](@ref win_flasher)
 * and/or [gdb server](@ref win_gdbservers) sections. Finally to
 * debug the gdb included in the toolchain was used; this last step
 * is also valid under Windows.
 *
 * ## Eclipse {#win_eclipse}
 *
 * Eclipse IDE allow the user to create a Makefile based project.
 * Starting with v0.2 development has been done with Eclipse, offering
 * user-friendly integrated features like CVS and debugging.
 * Still, the use of an IDE is optional. Makefile building system is still
 * at the core.
 * Some fine-tuning were necessary to make the debugger work inside Eclipse.
 *
 * ## KEIL MDK uVision4 {#win_keil}
 *
 * Even though KEIL's IDE doesn't support Makefile projects, HapticLib developers
 * added support to uVision4 enabling development of haptic application with this
 * tool.
 * A ready-to-use uVision4 workspace is
 * [available here](https://www.guardati.it/hapticLib/workspace_keil.zip) with all
 * the demo application of HapticLib project.
 *
 * @warning The IDE version tested is v4.60. Please make sure to have this version
 * of KEIL's uVision installed along with [ST-LINK Utility](@ref win_flasher) from STM.
 *
 * @note HapticLib main development take place within a different platform than KEIL's.
 * Some features are not available out-of-the-box using uVision4 IDE.
 * It is still possible though to have DBG and REL versions of compiled applications.
 *
 *
 * [win_make]: http://sourceforge.net/projects/gnuwin32/files/make/3.81/make-3.81.exe/download
 * [win_mingw]: http://www.mingw.org/wiki/Getting_Started
 * [win_cygwin]: http://cygwin.com/install.html
 * [win_linaro]: https://launchpad.net/gcc-arm-embedded/+milestone/4.6-2012-q2-update
 * [win_atollic]: http://www.atollic.com/index.php/download/truestudio-for-arm
 * [win_codesourcery]: http://www.mentor.com/embedded-software/sourcery-tools/sourcery-codebench/editions/lite-edition/
 * [win_st_utility]: http://www.st.com/internet/com/SOFTWARE_RESOURCES/TOOL/DEVICE_PROGRAMMER/stm32_st-link_utility.zip
 *
 *
 */

/**
 * @page lin_info Linux
 *
 * @note This page is directed to developers of haptic applications
 * on embedded platforms.
 *
 * Here is explained how to setup the development environment
 * in different configurations on **Linux** Operating System.
 *
 * [TOC]
 *
 * # Preliminary Setup {#lin_requirements}
 *
 * Before start, make sure the system provide basic GNU development
 * tools like autotools. Most linux distributions include such tools.
 * Target code compilation is managed using Makefiles that, along
 * with the tool-chain, is the only tool needed to produce code.
 * However, in order to debug application, a gdb-server interfacing
 * with the Hardware target device is needed.
 * To produce code documentation a recent version of doxygen is used.
 * The following section will explain all the single components used
 * in the development work flow.
 *
 * # Tool-chains supported {#toolchains}
 *
 * The tool-chain used in HapticLib development is linaro gcc targeting
 * ARM Cortex-M3 platform, compiled locally using a user-friendly
 * script.
 *
 * ## ARM {#arm}
 *
 * At the moment the ARM platform is the only one supported. In particular
 * the only physical target board used has a Cortex-M3 MCU.
 * Any gcc-derived toolchain should work without problem, but only linaro
 * tool-chain has been tested.
 *
 * ### linaro {#linaro}
 *
 * The linaro gcc tool-chain has been locally compiled using the building
 * script [summon-arm-toolchain](https://github.com/esden/summon-arm-toolchain).
 * It just download all the software components from their respective web-sites
 * and then compiles everything inside a target directory holding the toolchain
 * (including newlib C library and gdb).
 * At the end of the building process, you can just add the resulting bin/ path
 * to the system $PATH and the tool-chain's tools will be available.
 *
 * ### codesourcery {#codesourcery}
 *
 * This toolchain has not been tested, but it should work just fine.
 *
 * ## MSP430 {#msp430}
 *
 * At the moment the library dooesn't support this embedded platform.
 * Probably there should not be any problem porting HapticLib to it.
 *
 * ## PIC {#pic}
 *
 * At the moment the library dooesn't support this embedded platform.
 * Probably there should not be any problem porting HapticLib to it.
 *
 *
 * # GDB Servers supported {#gdbservers}
 *
 * The gdb server interfacing with the only dev board (STM32VLDISCOVERY)
 * used during HapticLib development was
 * [texane/stlink](https://github.com/texane/stlink).
 * It is an open source gdb server capable of flashing/erasing flash
 * memory, and debugging running code using stlink protocol with the
 * device and gdb protocol with the gdb client from the toolchain.
 *
 * # Flasher utilities {#flasher}
 *
 * To flash binary code to the target device (STM32VLDISCOVERY)
 * [texane/stlink](https://github.com/texane/stlink) was used.
 *  (see GDB server section).
 *
 * # IDE supported {#ides}
 *
 * ## No IDE {#noide}
 *
 * The library is versatile enough to be used without a complete
 * IDE; in fact you can compile the application and the library
 * (including this documentation) using the provided Makefiles.
 *
 * The first versions of HapticLib with the demo applications (up to v0.2)
 * were developed on linux using only the [vim](http://www.vim.org/)
 * text editor. The code was then compiled using
 * [GNU make](http://www.gnu.org/software/make/) which in turns
 * uses the gcc toolchain to produce the binary code. To flash the
 * code on the device, [texane/st-link](https://github.com/texane/stlink/)
 * was used. Finally to debug the gdb included in the toolchain was used.
 *
 * ## Eclipse {#eclipse}
 *
 * Eclipse IDE allow the user to create a Makefile based project.
 * Starting with v0.2 development has been done with Eclipse, offering
 * user-friendly integrated features like CVS and debugging.
 * Still, the use of an IDE is optional. Makefile building system is still
 * at the core.
 * Some fine-tuning were necessary to make the debugger work inside Eclipse.
 *
 *
 */

/**
 * @page dev_guide Developer Guide
 *
 * [TOC]
 *
 * In this page are explained internal details of HapticLib.
 * The Developer that wants to extend or modify the library
 * will find this page useful.
 *
 * # SystemDesc system descriptor. {#systemDesc}
 *
 * SystemDesc is a global structure holding the references
 * of the application runtime state.
 *
 * It holds a reference for each initialized haptor and
 * a reference for each initialized pattern.
 *
 * Using this variable all library code can interact with
 * the rest of the system easily.
 *
 * Additionally, here are stored the PWM driving signal
 * frequency and the global inter-samples delay.
 *
 * The scope of this variable would be global, but a
 * preprocessor conditionally inclusion has been made
 * to avoid the exposure of this variable to the user
 * application code.
 *
 * If the user needs access to this system variable, he
 * must define the HL_SYSTEM_FILE preprocessor symbol on
 * the module using it.
 *
 * The actual structure definition of system_desc is the
 * following:
 *
 * ~~~~~~~~~~~~~~~~~
 *
 * typedef struct system_desc {
 *	uint8_t              num_haptors;
 *	haptor_desc          haptors[MAX_HAPTORS];
 *	pattern_desc         patterns[MAX_HAPTORS];
 *	uint32_t             pwm_freq;
 *	uint8_t              samples_delay;
 * } system_desc;
 *
 * ~~~~~~~~~~~~~~~~~
 *
 * MAX_HAPTORS is a platform specific definition to
 * limit the actual maximum number of haptors available.
 *
 * @note Using static arrays to hold patterns and haptors
 *       references is not a smart way to use the memory
 *       resource.
 *       This way, irrespective of the actual number of
 *       haptors/patterns initialized, the maximum number
 *       of references is hold in memory.
 *       Future versions may include a dynamic memory
 *       allocator function, or directly link to a C library
 *       offering this kind of features (like the
 *       [newlib](http://sourceware.org/newlib/) C library).
 *
 * # Haptor Descriptor {#haptorDesc}
 *
 * SystemDesc holds a reference to each haptor initialized.
 * The reference is a descriptor holding the following
 * information about the haptor:
 *
 * ~~~~~~~~~~~~~~~
 *
 *  typedef struct haptor_desc {
 *	 uint8_t             id;
 *	 uint16_t            min_duty;
 *	 uint16_t            max_duty;
 *	 pattern_desc       *activePattern;
 *	 struct haptor_desc *nextHaptor;
 *	} haptor_desc;
 * ~~~~~~~~~~~~~~~
 *
 * min_duty and max_duty are useful to calibrate the range
 * of operation of the physical device.
 *
 * activePattern refer to the descriptor of the pattern
 * attached to this haptor (if any).
 *
 * nextHaptor is a reference to another haptor. This pointer
 * creates a linked list of haptors belonging to the same group.
 * It is used to implement multi-haptor patterns. (see below for
 * further informations)
 *
 * # Pattern Descriptor {#patternDesc}
 *
 * SystemDesc hold a reference to each active pattern in the running
 * system.
 *
 * The reference is a descriptor of a pattern holding the following
 * data:
 *
 * ~~~~~~~~~~~~
 *
 * typedef struct pattern_desc {
 *   pattern_name	    name;
 *   user_param	       *userParams;
 *   status_param	    statusParams;
 *   pattern_continuator continuator;
 *   struct haptor_desc *activeHaptorList;
 * } pattern_desc;
 *
 * ~~~~~~~~~~~~
 *
 * pattern_name is a numeric id (int) handled as enumeration
 * ( pattern_name ) to make code cleaner and easy to read.
 *
 * userParams is a reference to a structure holding the user
 * provided parameters needed by the pattern.
 *
 * @note user_param is defined by the pattern developer.
 *       The user application must conform to it.
 *       The actual variable holding this data is declared
 *       by the user in one of its modules ( generally main.c )
 *       HapticLib just refers to it. ( refer below to
 *       understand why )
 *
 * statusParams is a data structure holding the state of the
 * pattern. This structure is defined by the pattern developer
 * and is not exposed to the user.
 *
 * continuator is a function pointer to the code that will be
 * executed at every scheduled instant by this active pattern.
 * If this pointer is NULL, then the system will consider this
 * pattern not active.
 *
 * the continuator is a function defined by the pattern developer
 * and is not exposed to the user.
 *
 * activeHaptorList points to the first haptor of the linked list
 * forming the group of attached haptors.
 * Using this list a pattern can easily developed as multi-haptor.
 *
 * # Pattern Rendering {#patterns}
 *
 * When a pattern is initialized, an available slot on SystemDesc
 * is searched for and, if found, the pattern goes there.
 *
 * The pattern now has no haptor attached to it, so the user must
 * add every haptor to it. At every call of hl_addHaptor() the
 * linked-list is formed for later references by the pattern code.
 *
 * Now the pattern is ready to be started for the actual rendering.
 *
 * The rendering of a pattern has two main steps:
 *
 *    - initiator
 *    - continuator
 *
 * When the pattern is started, a function called pattern
 * initiator is called to initialize the pattern initial status.
 * userParams is used to set the initial status of the pattern
 * that is hold in statusParams.
 *
 * The initiator must set the pattern continuator on its pattern
 * descriptor.
 *
 * This operation is the actual start of the pattern, as the
 * patternScheduler check for this pointer to be not-NULL to
 * execute the pattern continuator code.
 *
 * From now on, when the scheduler decides, the pattern continuator
 * is executed.
 *
 * The actual logic inside the continuator, is up to the pattern
 * developer.
 *
 * Usually the task carried by the continuator are:
 *
 *  - read the user parameters for contingent run-time events.
 *  - update the status to reflect the time flow.
 *  - implement an exit strategy
 *
 * The userParams polling is a key features to let the user
 * drive the pattern logic ( as coded by the pattern developer ).
 *
 * The exit strategy is a way for the pattern to self-deallocate
 * from scheduling. This is not mandatory, but the pattern developer
 * must explicitly document the behavior.
 * For example, a pattern could self-stop after some time of execution.
 * Another pattern could run indefinitely and stop only when the user
 * forces its stop calling hl_stopPattern().
 *
 *
 * # Develop a New Pattern Generator {#new_pattern}
 *
 * To create a new pattern generator, the easy way is to copy an existing
 * one and the modify it.
 *
 * Here are the steps to implement a new test pattern generator:
 *
 *  1. Create a new directory inside HapticLib/patterns: test2
 *  2. Create test2.c and test2.h source code files.
 *  3. add the new pattern in hl_patterns.c / hl_patterns.h
 *
 * test2.h will contain the definitions of the new pattern:
 *
 *  - test2StatusParameters
 *  - test2UserParameters (optional)
 *  - other optional structures or definitions needed by the pattern.
 *
 * test2.c will contain at least initiator and continuator code:
 *
 *  - test2PatternGenerator(): the initiator
 *  - test2Continuator(): the continuator
 *  - Doxygen embedded documentation to describe the pattern.
 *
 * Be sure to implement a correct exit strategy on the continuator
 * triggered by some event (could be a value on some userParam set
 * by the user application code), or explicitly state on pattern
 * documentation that the user must call hl_stopPattern() to stop
 * it.
 *
 * The documentation template is as follow:
 *
 *  - Introduction
 *  - Theory
 *  - Optional Parameters
 *  - Usage Examples
 *  - Debugging Details
 *  - Additional Notes
 *
 * Inside hl_patterns.h, references to the new pattern must be added
 * in 4 points:
 *
 *  - add an include to the header: \#include "test2.h"
 *  - update pattern_name enumeration with the new pattern name
 *    before the last entry: Num_Patterns_Available.
 *  - update status_param union to contain the new statusParams.
 *  - update user_params (if any) to contain the new userParams.
 *
 * Inside hl_patterns.c, the new pattern must be added in 2 points:
 *
 *  - define a new extern pattern initiator
 *  - add the pointer to the patternMap
 *
 *
 * # Adding a New Platform {#new_platform}
 *
 * Adding a new platform, like for patterns, consists in adding
 * a new sub-directory inside HapticLib/platforms.
 *
 * @warning As for now ( HapticLib v.0.7 ) a cleaner interface
 *          between the platform specific module and the rest
 *          of the library need to be defined. There are some
 *          direct references inside hapticLib.c that are
 *          platform specific and need to be moved.
 *
 * # HapticLib vs. bare application comparison {#bare_comparison}
 *
 * Comparison of HapticLevel demo application with a modified
 * version of MEMS demo application from STM.
 *
 * Instructions for HapticLevel compilation:
 *
 *  - Download the workspace_keil.zip environment
 *  - Launch Keil project for HapticLevel demo
 *  - Set "REL" target, and compile
 *  - use arm-none-eabi-objcopy -I ihex HapticLevel.hex -O binary HapticLevel.bin
 *  - HapticLevel.bin is produced
 *
 * Instructions for modified demo compilation:
 *
 *  - Download original package from STM [here](http://www.st.com/internet/com/SOFTWARE_RESOURCES/SW_COMPONENT/FIRMWARE/stm32f4discovery_fw.zip)
 *  - Download diff patch [here](https://www.guardati.it/hapticLib/bare_app.patch)
 *  - Unzip and go into the unzipped directory
 *  - Apply patch: patch -p1 < ../bare_app.patch
 *  - Start project using Keil project located in STM32F4-Discovery_FW_V1.1.0/Project/Peripheral_Examples/MEMS/MDK-ARM and compile.
 *  - launch arm-none-eabi-objcopy -I ihex LIS302DL.hex -O binary LIS302DL.bin
      inside STM32F4-Discovery_FW_V1.1.0/Project/Peripheral_Examples/MEMS/MDK-ARM/LIS302DL dir
 *  - LIS302DL.bin  is produced
 *
 *
 */

/**
 * @page references References
 *
 * Here there are links of all the references of HapticLib.
 *
 * [ref_example]: https://guardati.it
 * [impact_article_1]: http://ieeexplore.ieee.org/xpl/articleDetails.jsp?tp=&arnumber=951362&contentType=Journals+%26+Magazines&sortType%3Dasc_p_Sequence%26filter%3DAND%28p_IS_Number%3A20572%29
 * [impact_article_2]: http://ieeexplore.ieee.org/xpl/articleDetails.jsp?tp=&arnumber=5759662&contentType=Conference+Publications&sortType%3Dasc_p_Sequence%26filter%3DAND%28p_IS_Number%3A5759580%29%26pageNumber%3D4
 * [win_make]: http://sourceforge.net/projects/gnuwin32/files/make/3.81/make-3.81.exe/download
 * [win_mingw]: http://www.mingw.org/wiki/Getting_Started
 * [win_cygwin]: http://cygwin.com/install.html
 * [win_linaro]: https://launchpad.net/gcc-arm-embedded/+milestone/4.6-2012-q2-update
 * [win_atollic]: http://www.atollic.com/index.php/download/truestudio-for-arm
 * [win_codesourcery]: http://www.mentor.com/embedded-software/sourcery-tools/sourcery-codebench/editions/lite-edition/
 * [win_st_utility]: http://www.st.com/internet/com/SOFTWARE_RESOURCES/TOOL/DEVICE_PROGRAMMER/stm32_st-link_utility.zip
 *
 *
 * - [Example Reference Text][ref_example]
 * - [Hachisu et al - Pseudo-haptic feedback augmented with visual and tactile vibrations][impact_article_1]
 * - [Okamura et al - Reality-based models for vibration feedback in virtual environments][impact_article_2]
 * - [GNU make.exe][win_make]
 * - [MinGW][win_mingw]
 * - [Cygwin][win_cygwin]
 * - [Linaro - GCC ARM Embedded][win_linaro]
 * - [Atollic - TrueSTUDIO for ARM Lite][win_atollic]
 * - [Mentor Graphics - Sourcery CodeBench Lite Edition][win_codesourcery]
 * - [ST Microelectronics - ST-LINK Utility][win_st_utility]
 *
 */

#include <stdint.h>
#include <stdlib.h>

#ifdef HL_DEBUG
#include "hl_debug.h"
#endif

#include "hl_patterns.h"

/**
 * @brief  Define this symbol to **enable the Debugging Features**.
 *
 * The Debugging features offered by the library are:
 *  - send_string()
 *  - send_int()
 *
 * The HapticLib Debugging Features are enabled based on the presence of the
 * symbol @ref HL_DEBUG .
 *
 * To define the symbol do either:
 * 		- explicitly \#`define HL_DEBUG` inside a module.
 * 		- add `-DHL_DEBUG` to the compiler command line during compilation.
 *
 * @warning Always make calls to debugging code inside conditionally
 * 			included blocks.
 * ~~~~~~~~~~~~~~~~
 *
 *			...
 *			#ifdef HL_DEBUG
 *				...
 *				send_int( 1234567 );
 *				...
 *				send_string("Correct usage.\n\r\0");
 *				...
 *			#endif
 *			...
 *
 * ~~~~~~~~~~~~~~~~
 *
 */
#ifndef HL_DEBUG
#define HL_DEBUG
#undef HL_DEBUG
#endif

/**
 * @brief   This symbol define what platform we are building for.
 *
 * This symbol is needed by the **Platform Specific Module** to
 * select the right low level code implementations.
 *
 * @note You MUST define this symbol using the compiler command line.
 *
 */
#ifndef STM32VLDISCOVERY
#define STM32VLDISCOVERY
#endif

/*
 * Platform Specific Hardware Level
 */
#ifdef STM32VLDISCOVERY
#include "hl_STM32VLDISCOVERY.h"
#elif OTHERS
#include "hl_OTHERS.h"
#endif

/**
 * @brief Single haptor descriptor
 *
 * HapticLib keep track of all the haptors configured in the system
 * using one @ref haptor_desc struct for each of them.
 *
 * To know what kind of informations holds, please refer to its
 * data structure @ref haptor_desc.
 *
 */
struct haptor_desc;

 /**
  * @brief Single haptor descriptor
  *
  * HapticLib keep track of all the haptors configured in the system
  * using one @ref haptor_desc struct for each of them.
  *
  * To know what kind of informations holds, please refer to its
  * data structure @ref haptor_desc.
  *
  */
 typedef struct haptor_desc {
	 uint8_t             id;			/**< @brief Unique ID to
	 	 	 	 	 	 	 	 	 	 	 designate the haptor.
	 	 	 	 	 	 	 	 	 	 	 @note This is a
	 	 	 	 	 	 	 	 	 	 	 temporary solution
	 	 	 	 	 	 	 	 	 	 	 waiting for a cleaner
	 	 	 	 	 	 	 	 	 	 	 **User API** <-->
	 	 	 	 	 	 	 	 	 	 	 **Platform Specific**
	 	 	 	 	 	 	 	 	 	 	 interface to be
	 	 	 	 	 	 	 	 	 	 	 defined.*/
	 uint16_t            min_duty;		/**< @brief Minimum duty
 	 	 	 	 	 	 	 	 	 	 	 cycle to activate the
 	 	 	 	 	 	 	 	 	 	 	 tactor. */
 	 uint16_t            max_duty;		/**< @brief Maximum duty
 	 	 	 	 	 	 	 	 	 	 	 cycle to feed to the
 	 	 	 	 	 	 	 	 	 	 	 tactor. */
 	 pattern_desc       *activePattern;	/**< @brief Reference to the
 	  	  	  	  	  	  	  	  	  	  	 active pattern driving
 	  	  	  	  	  	  	  	  	  	  	 this haptor.

 	  	  	  	  	  	  	  	  	  	  	 If it is `NULL` the
 	  	  	  	  	  	  	  	  	  	  	 haptor is free to be
 	  	  	  	  	  	  	  	  	  	  	 activated by a
 	  	  	  	  	  	  	  	  	  	  	 pattern. */

 	 struct haptor_desc *nextHaptor;	/**< @brief Reference to
 	 	 	 	 	 	 	 	 	 	 	 another
 	 	 	 	 	 	 	 	 	 	 	 @ref haptor_desc.

 	 	 	 	 	 	 	 	 	 	 	 This links create a
 	 	 	 	 	 	 	 	 	 	 	 list of haptors chained
 	 	 	 	 	 	 	 	 	 	 	 together to form an
 	 	 	 	 	 	 	 	 	 	 	 **haptor group** driven
 	 	 	 	 	 	 	 	 	 	 	 by the same pattern. */
 } haptor_desc;


#ifdef HL_SYSTEM_FILE
 /**
  * @brief System configuration descriptor.
  *
  * A global variable [SystemDesc](@ref SystemDesc) is kept at
  * runtime to let the system know the status of the haptic
  * system. All the active haptors and active patterns are
  * described here.
  *
  */
 typedef struct system_desc {
 	uint8_t              num_haptors;	      /**< @brief Total
 						                        number of haptors
 						                        configured in the
 						                        system.

 						                        This number can be
 						                        less than MAX_HAPTOR
 						                        if the user
 						                        configure the system
 						                        so (calling
 						                        hl_configure() ).

 						                        This can be useful
 						                        if the haptic
 						                        application need
 						                        some hardware
 						                        resources reserved
 						                        for other uses than
 						                        haptor driving. */
 	haptor_desc          haptors[MAX_HAPTORS]; /**< @brief Array
 	 	 	 	 	 	 	 	 	 	 	 	of descriptors for
 	 	 	 	 	 	 	 	 	 	 	 	every single haptor
 	 	 	 	 	 	 	 	 	 	 	 	configured.

 	 	 	 	 	 	 	 	 	 	 	 	When the user calls
 	 	 	 	 	 	 	 	 	 	 	 	hl_configure(), this
 	 	 	 	 	 	 	 	 	 	 	 	array is initialized
 	 	 	 	 	 	 	 	 	 	 	 	and indexed up to
 	 	 	 	 	 	 	 	 	 	 	 	@ref num_haptors.

 	 	 	 	 	 	 	 	 	 	 	 	@note At the moment
 	 	 	 	 	 	 	 	 	 	 	 	a dynamic allocation
 	 	 	 	 	 	 	 	 	 	 	 	of @ref num_haptors
 	 	 	 	 	 	 	 	 	 	 	 	structures is not
 	 	 	 	 	 	 	 	 	 	 	 	supported. So we
 	 	 	 	 	 	 	 	 	 	 	 	need to statically
 	 	 	 	 	 	 	 	 	 	 	 	allocate the
 	 	 	 	 	 	 	 	 	 	 	 	maximum. This will
 	 	 	 	 	 	 	 	 	 	 	 	be solved in future
 	 	 	 	 	 	 	 	 	 	 	 	version. */
 	pattern_desc         patterns[MAX_HAPTORS]; /**< @brief Array
 												of descriptors for
 												all the patterns
 												that could be
 												running in the
 												system.

 												There cannot be more
 												than num_haptors
 												patterns active at
 												any given time.

 	 	 	 	 	 	 	 	 	 	 	 	@note At the moment
 	 	 	 	 	 	 	 	 	 	 	 	a dynamic allocation
 	 	 	 	 	 	 	 	 	 	 	 	of @ref num_haptors
 	 	 	 	 	 	 	 	 	 	 	 	structures is not
 	 	 	 	 	 	 	 	 	 	 	 	supported. So we
 	 	 	 	 	 	 	 	 	 	 	 	need to statically
 	 	 	 	 	 	 	 	 	 	 	 	allocate the
 	 	 	 	 	 	 	 	 	 	 	 	maximum. This will
 	 	 	 	 	 	 	 	 	 	 	 	be solved in future
 	 	 	 	 	 	 	 	 	 	 	 	version. */
 	uint32_t             pwm_freq;	          /**< @brief PWM
 												Frequency (in Hz)
 						  	  	  	  	  	    used across all the
 						  	  	  	  	  	    haptors.

 								                This value is used
 								                by the **Platform
 								                Specific** module
 								                code to setup the
 								                Time base
 								                reference values. */
 	uint8_t              samples_delay;       /**< @brief PWM
 												inter-samples
 								                delay (in ms).

 								                This value is used
 								                by the **Platform
 								                Specific** module
 								                code to call the
 								                patternScheduler()
 								                every `samples_delay`
 								                ms. */
 } system_desc;
#endif

haptor_desc  *hl_configure(uint32_t, uint8_t, uint8_t);
pattern_desc *hl_initPattern(pattern_name, user_param *);
pattern_desc *hl_addHaptor(haptor_desc *, pattern_desc *);
uint8_t       hl_startPattern(pattern_desc *);
uint8_t       hl_stopPattern(pattern_desc *);

