/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file hl_debug.c
 *
 * @brief **Debugging Features** definitions.
 *
 * This file is part of the **HapticLib Debugging Features**
 * module.
 *
 * Other HapticLib internal modules use the tools implemented
 * including the file hl_debug.h.
 *
 * @note To enable the compilation of this code, the symbol
 *       @ref HL_DEBUG must be defined, usually passed to
 *       the compiler command line as: `-DHL_DEBUG`
 *
 * The features implemented are:
 *  - send_string()
 *  - send_int()
 *
 * @note User applications can use these functions in their code
 *       without directly include hl_debug.h.
 *
 * @author Leonardo Guardati
 * @version 0.7
 * @date 2012
 *
 */

/**
 * @brief  Define this symbol to **enable the Debugging Features**.
 *
 * The Debugging features offered by the library are:
 *  - send_string()
 *  - send_int()
 *
 * The HapticLib Debugging Features are enabled based on the presence of the
 * symbol @ref HL_DEBUG .
 *
 * To define the symbol do either:
 * 		- explicitly \#`define HL_DEBUG` inside a module.
 * 		- add `-DHL_DEBUG` to the compiler command line during compilation.
 *
 * @warning Always make calls to debugging code inside conditionally
 * 			included blocks.
 * ~~~~~~~~~~~~~~~~
 *
 *			...
 *			#ifdef HL_DEBUG
 *				...
 *				send_int( 1234567 );
 *				...
 *				send_string("Correct usage.\n\r\0");
 *				...
 *			#endif
 *			...
 *
 * ~~~~~~~~~~~~~~~~
 *
 */
#ifndef HL_DEBUG
#define HL_DEBUG
#undef  HL_DEBUG
#endif

#include "hl_debug.h"

#ifdef HL_DEBUG
/**
 * @brief **DEBUG function** to print over the USART any *uint32_t*.
 *
 * @warning Only if the [HL_DEBUG](@ref HL_DEBUG) symbol has been
 *          defined, the user can call this function.
 *
 * send_int() will send the **hexadecimal representation** of
 * any *uint32_t*.
 *
 * @warning Always make calls to send_int() inside conditionally
 * 			included blocks.
 *
 * ~~~~~~~~~~~~~~~~
 *          #ifdef HL_DEBUG
 *              send_int( 1234567 );
 *          #endif
 * ~~~~~~~~~~~~~~~~
 *
 * If the value passed is less then 100000, send_int() will also
 * send the **decimal representation** inside brackets.
 * ~~~~~~~~~~~~~~~~
 * 		...
 * 		#ifdef HL_DEBUG
 * 		    ...
 * 		    send_int( 12345 );  // outputs -->   0x3039 (12345)
 * 		    ...
 * 		#endif
 * 		...
 * ~~~~~~~~~~~~~~~~
 *
 * @note Implementing this function, dependency on other
 *       libraries is avoided.
 *
 * @param[in] val `uint32_t` variable containing the value to
 * 			      print.
 *
 * @return The sent value is returned.
 *
 */
uint32_t send_int(uint32_t val)
{
	uint8_t i=0;
	uint8_t front_pad_skip = 1;
	uint8_t str_val[8] = { 0,0,0,0,0,0,0,0 };

	// Print in hex format.
	for( i=0 ; i<8 ; i++ )
	{
		str_val[i] = (uint8_t) ( ( val >> (4*(7-i)) ) & 0xF ) ;
		str_val[i] += ( str_val[i] > 9  ) ? ( 65 - 10 ) : 48 ;
	}

	send_char('0');
	send_char('x');

	for(i=0;i<8;i++)
	{
		if( front_pad_skip && str_val[i] == '0' && i != 7)
		{
			//Skip useless 0x00000...zeros
		}
		else {
			send_char( str_val[i] );
			front_pad_skip = 0;
		}

		str_val[i] = 0;
	}

	// For small number, print the decimal value too.
	if( val < 10 )
		str_val[0] = (uint8_t) ( val + 48 );
	else if ( val < 100 )
	{
		str_val[0] = (uint8_t) ( val/10 + 48 );
		str_val[1] = (uint8_t) ( val%10 + 48 );
	}
	else if ( val < 1000 )
	{
		str_val[0] = (uint8_t) (  val/100     + 48 );
		str_val[1] = (uint8_t) ( (val%100)/10 + 48 );
		str_val[2] = (uint8_t) ( (val%100)%10 + 48 );
	}
	else if ( val < 10000 )
	{
		str_val[0] = (uint8_t) (   val/1000          + 48 );
		str_val[1] = (uint8_t) (  (val%1000)/100     + 48 );
		str_val[2] = (uint8_t) ( ((val%1000)%100)/10 + 48 );
		str_val[3] = (uint8_t) ( ((val%1000)%100)%10 + 48 );
	}
	else if (val < 100000 )
	{
		str_val[0] = (uint8_t) (    val/10000                + 48 );
		str_val[1] = (uint8_t) (   (val%10000)/1000          + 48 );
		str_val[2] = (uint8_t) (  ((val%10000)%1000)/100     + 48 );
		str_val[3] = (uint8_t) ( (((val%10000)%1000)%100)/10 + 48 );
		str_val[4] = (uint8_t) ( (((val%10000)%1000)%100)%10 + 48 );
	}

	send_char(' ');
	send_char('(');
	for(i=0;i<8;i++)
	{
		send_char( str_val[i] );
		str_val[i] = 0;
	}
	send_char(')');

	return val;
}

/**
 * @brief **DEBUG function** to print over the USART any string.
 *
 * send_string() will send over the serial link any NULL
 * terminated string (`char *`) smaller than 255 characters.
 *
 * @warning Only if the [HL_DEBUG](@ref HL_DEBUG) symbol has been
 *          defined, the user can call this function.
 *
 * @warning Always make calls to send_string() inside conditionally
 * 			included blocks.
 *
 * @warning Also REMEMBER TO APPEND THE NULL TERMINATING CHARACTER
 *          TO THE STRINGS PASSED!!!
 * ~~~~~~~~~~~~~~~~
 *          #ifdef HL_DEBUG
 *               send_string("Correct usage.\n\r\0");
 *          #endif
 * ~~~~~~~~~~~~~~~~
 *
 * @note The 255 character limit is arbitrary and only for safety
 *       in case you forget to append the '\0' character.
 * 	     If you need to send more, call again send_string().
 *
 * @param[out] string `char*` holding the reference to the string
 * 					  to be printed.
 *
 */
void send_string(char *string)
{
	uint8_t i=0;
	while( string[i] != 0 && i < 255 )
		send_char(string[i++]);
}
#endif
