/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file hl_debug.h
 *
 * @brief **Debugging Features** header.
 *
 * This file is part of the **HapticLib Debugging Features**
 * module.
 *
 * It is included by other HapticLib internal modules to use
 * the tools implemented.
 *
 * @note To enable the compilation of this code, the symbol
 *       @ref HL_DEBUG must be defined, usually passed to
 *       the compiler command line as: `-DHL_DEBUG`
 *
 * The features implemented are:
 *  - send_string()
 *  - send_int()
 *
 * @note User applications can use these functions in their code
 *       without directly include hl_debug.h.
 *
 * @author Leonardo Guardati
 * @version 0.7
 * @date 2012
 */

#include <stdint.h>

/*
 * Platform Specific Hardware Level
 */
#ifdef STM32VLDISCOVERY
#include "hl_STM32VLDISCOVERY.h"
#elif OTHERS
#include "hl_OTHERS.h"
#endif

/**
 * @brief  Define this symbol to **enable the Debugging Features**.
 *
 * The Debugging features offered by the library are:
 *  - send_string()
 *  - send_int()
 *
 * The HapticLib Debugging Features are enabled based on the presence of the
 * symbol @ref HL_DEBUG .
 *
 * To define the symbol do either:
 * 		- explicitly \#`define HL_DEBUG` inside a module.
 * 		- add `-DHL_DEBUG` to the compiler command line during compilation.
 *
 * @warning Always make calls to debugging code inside conditionally
 * 			included blocks.
 * ~~~~~~~~~~~~~~~~
 *
 *			...
 *			#ifdef HL_DEBUG
 *				...
 *				send_int( 1234567 );
 *				...
 *				send_string("Correct usage.\n\r\0");
 *				...
 *			#endif
 *			...
 *
 * ~~~~~~~~~~~~~~~~
 *
 */
#ifndef HL_DEBUG
#define HL_DEBUG
#undef  HL_DEBUG
#endif

#ifdef HL_DEBUG
extern uint8_t send_char(uint8_t ch);
uint32_t send_int(uint32_t);
void     send_string(char *);
#endif
