/*
 * Copyright (c) 2012, Silvio Vallorani <silvio.vallorani@studio.unibo.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file constant.c
 *
 * @brief **Constant Pattern** generator function definition
 *
 *
 * # Introduction
 *
 * Constant Pattern can be used to shake haptors providing
 * the shaking magnitude.
 *
 * # Theory
 *
 * Simply a on/off pattern.
 *
 * Link to optional tools used on pattern development.
 *
 * No other tools needed.
 *
 * # Optional Parameters
 *
 * User must provide a constant value from 0 to 65000:
 * 	- 1 to 65000 means set an haptor in busy mode and shake it
 * 	at provided value.
 *
 * # Usage Examples
 *
 * ~~~~~~~~~~~~~~~
 * ...
 * //start one haptor shaking @ 35000
 * haptor_desc *myHaptor = hl_configure(24000000,15,1);
 *
 * user_param myParams; // param must use struct of user_param_t type
 * myParams.constant.constant = 35000;
 *
 * pattern_desc *myConstantPattern = hl_initPattern(Constant,&myParams);
 *
 * hl_addHaptor(myHaptor,myConstantPattern);
 *
 * hl_startPattern(myConstantPattern);
 * ...
 * hl_stopPattern(myConstantPattern);
 * ...
 * ~~~~~~~~~~~~~~~
 *
 * # Debugging Details
 *
 * Parameters can't be NULL. It NULL is passed return an error.
 * If HL_DEBUG symbol defined a "NULL found" string will sent to UART.
 *
 * # Additional Notes
 *
 * The `constant` parameter can be used to change the duty-cycle even
 * after the pattern has started!
 * At ever continuator pass, it will poll the actual value of the user
 * parameter `constant` and update the duty-cycle accordingly.
 *
 * @warning When use Constant Pattern remember to free an haptor that
 * was previously started using this pattern.
 *
 * @note To stop this pattern and free its active haptor, use
 *       hl_stopPattern() with the right pattern descriptor passed
 *       as argument.
 *
 * @authors Leonardo Guardati
 * @authors Silvio Vallorani
 * @version v0.7
 * @date 2012
 *
 */

#include "hapticLib.h"

/**
 * @brief Pattern generator **continuator**
 *
 * This function is called back from the patternScheduler() every
 * [SystemDesc](@ref SystemDesc).samples_delay ms.
 *
 * At every call, this continuator will decide what to do.
 *
 * For a simple and static pattern, the action could be to simply
 * point to the next element of an array of samples, or to generate
 * it on the fly through a certain formula.
 *
 * For more complex scenarios, this pattern continuator callback
 * should be able to deliver a good degree of flexibility.
 *
 * @warning The **continuator** must implement an exit strategy to
 * 			decide when the pattern is finished an remove itself
 * 			from the scheduling patterns. If this is not done right,
 * 			the pattern will never free the haptor to other patterns.
 *
 * @param[out] pattern A @ref pattern_desc* holding the reference to
 *                     the running pattern instance to work with.
 *                     Through this reference, the continuator can
 *                     extract the user and status parameters in
 *                     order to execute the haptic pattern.
 *
 * @return uint8_t  The result is returned to indicate success
 * 					or error.
 *
 * @retval 0 Success
 *
 * @retval 1 Error
 *
 */
uint8_t constantContinuator(pattern_desc *pattern)
{

	/*
	 * Create a handy reference for userParameters and statusParameters
	 * Note:
	 * 		- userParams data reside on the user program module. (main.c)
	 * 		- statusParams data reside on SystemDesc variable.
	 */
	constantUserParameters     *userParams = &pattern->userParams->constant;
	constantStatusParameters *statusParams = &pattern->statusParams.constant;

	statusParams->duty = userParams->constant;

	TIM_Channel_DutyChanger( dutyConverter(statusParams->duty, pattern->activeHaptorList), \
								pattern->activeHaptorList->id );

	return 0;

}

/**
 * @brief Pattern generator **initiator**
 *
 * This function is called by the **User API Module** of HapticLib,
 * when the library user calls hl_startPattern().
 *
 * The first task for the pattern generator **initiator** is to
 * validate the user provided inputs, this cannot be done
 * anywhere else.
 *
 * This pattern initiator then sets up all the status parameters.
 *
 * If the pattern accept user parameters, the status setting can be
 * based on the user provided parameters.
 *
 * The last duty of this function is to put the pattern continuator
 * up for scheduling ( setting
 *    `pattern->continuator = `&`constantContinuator;`), so the
 * next time patternScheduler() will check for active patterns,
 * it will find this pattern.
 *
 * @warning If the **initiator** doesn't set the continuator up for
 *          scheduling, the pattern will never start, and for
 *          HapticLib, the haptor is free to receive other patterns.
 *
 * @param[out] pattern A @ref pattern_desc* holding the running
 *                     instance of the pattern being started.
 *
 * @return uint8_t  The result is returned to indicate success
 * 					or error.
 *
 * @retval 0 Success
 *
 * @retval 1 Error
 *
 */
uint8_t constantPatternGenerator(pattern_desc *pattern)
{
	/*
	 * Create a handy reference for userParameters
	 *                        and statusParameters
	 * Note:
	 * 		- userParams data reside on the user
	 * 					program module. (main.c)
	 * 		- statusParams data reside on SystemDesc
	 * 					variable.
	 */
	constantUserParameters     *userParams = &pattern->userParams->constant;
	constantStatusParameters *statusParams = &pattern->statusParams.constant;


	// User Parameters Validation:
	// Input validation must be made before attempting
	// to use the provided parameters.
	if( userParams == NULL )
	{
#ifdef HL_DEBUG
		send_string("constantPatternGenerator(): expecting optional Parameters, NULL found.\r\n\0");
#endif
		return 1;
	}

	// Status initialization based on user provided parameters...
	statusParams->duty = userParams->constant;


	// Last duty: Set the continuator up for scheduling.
	pattern->continuator  = &constantContinuator;

	return 0;
}
