/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file generic.c
 *
 * @brief **Generic Pattern** generator function definition [TEMPLATE]
 *
 * [TEMPLATE]
 *
 * This is a template file; copy and rename it as starting point
 * for the implementation of a new Pattern Generator.
 *
 * Follow the structure of this example pattern (code and
 * documentation formats) while implementing a new pattern generator.
 *
 * The **initiator** and the **continuator** need to be implemented
 * and documented. First read the documentation and the code of both
 * and the implement your own pattern code and document it.
 *
 * @note Remember to use the associated header file too.
 * 		 For the template is:
 * 			- generic.c: Documentation, initiator and continuator.
 * 			- generic.h: Pattern specific definitions, user and
 * 						 status parameters definitions
 *
 * @note For your pattern they become:
 *          - `mypattern.c`: Documentation, initiator and continuator.
 *          - `mypattern.h`: Pattern specific definitions, user and
 * 						     status parameters definitions
 *
 * [TEMPLATE]
 *
 * Implementation code of the Generic Pattern.
 *
 * # Introduction
 *
 * Description with main features only mentioned.
 *
 * # Theory
 *
 * Reference to theoretical documentation on which the pattern is
 * based.
 *
 * Link to optional tools used on pattern development.
 *
 * Description of main results.
 *
 * # Optional Parameters
 *
 * Detailed description of parameters needed and structure defined to
 * contain them.
 *
 * Parameters values ranges MUST be explained here.
 *
 * # Usage Examples
 *
 * Put some code snippets to illustrate how to use the pattern.
 *
 * # Debugging Details
 *
 * Explain (if any) the behavior of optional debugging features present
 * inside the generator.
 *
 * # Additional Notes
 *
 * Add some optional notes.
 *
 * @authors Leonardo Guardati
 * @authors Silvio Vallorani
 * @version v0.7
 * @date 2012
 *
 */

 /**
 * @brief Generator function **Debugging Capabilities**.
 *
 * The HapticLib Debugging Features are available on the
 * pattern generator function too.
 *
 * The Debugging Features are enabled based on the presence
 * of the symbol HL_DEBUG.
 *
 * The code must have conditional inclusion of debugging code.
 *
 * 		...
 * 		#ifdef HL_DEBUG
 * 			...
 * 			send_string("Debug Message...\n\r\0");
 * 			...
 * 		#endif
 * 		...
 *
 * @warning Unconditionally use of debugging code will result
 * in compiling errors when linking the release version of
 * the object files (compiled without -DHL_DEBUG passed to
 * the compiler).
 *
 */
/*
 * The following 4 preprocessor directives, are useless; they
 * do nothing. They are here to let doxygen generate documentation
 * about HL_DEBUG symbol behavior.
 * When implementing a new pattern from this file, it is recommended
 * to remove :
 *      - the doxygen comment above
 *      - this comment
 *      - the following 4 directives.
 *
 */
#ifndef HL_DEBUG
#define HL_DEBUG
#undef  HL_DEBUG
#endif

#include "hapticLib.h"

/**
 * @brief Pattern generator **continuator**
 *
 * This function is called back from the patternScheduler() every
 * [SystemDesc](@ref SystemDesc).samples_delay ms.
 *
 * At every call, this continuator will decide what to do.
 *
 * For a simple and static pattern, the action could be to simply
 * point to the next element of an array of samples, or to generate
 * it on the fly through a certain formula.
 *
 * For more complex scenarios, this pattern continuator callback
 * should be able to deliver a good degree of flexibility.
 *
 * @warning The **continuator** MUST implement an exit strategy to
 * 			decide when the pattern is finished an remove itself
 * 			from the scheduling patterns. If this is not done right,
 * 			the pattern will never free the haptor(s) for the other
 * 			patterns to use it(them) again.
 *
 * @param[out] pattern A @ref pattern_desc* holding the reference to
 *                     the running pattern instance to work with.
 *                     Through this reference, the continuator can
 *                     extract the user and status parameters in
 *                     order to execute the haptic pattern.
 *
 * @return uint8_t  The result is returned to indicate success
 * 					or error.
 *
 * @retval 0 Success
 *
 * @retval 1 Error
 *
 */
uint8_t genericContinuator(pattern_desc *pattern)
{
	haptor_desc* tmp = NULL;
	/*
	 * Create a handy reference for userParameters and statusParameters
	 * Note:
	 * 		- userParams data reside on the user program module. (main.c)
	 * 		- statusParams data reside on SystemDesc variable.
	 */
	genericUserParameters     *userParams = &pattern->userParams->generic;
	genericStatusParameters *statusParams = &pattern->statusParams.generic;

	uint16_t increment = 0;

	if( userParams->increment == smallIncrement )
		increment =  650;
	else if ( userParams->increment == bigIncrement )
		increment = 1300;
	else
		increment = 1000;

	if( statusParams->flag == 0 )
	{
		statusParams->duty += increment;

		if( statusParams->duty > 63000 )
			statusParams->flag = 1;
	}
	else
	{
		statusParams->duty -= increment;

		if( statusParams->duty < 2000 )
		{
			// Ensure finish without vibrations (multi-haptor)
			tmp = pattern->activeHaptorList;
			while( tmp )
			{
				TIM_Channel_DutyChanger( 0, tmp->id );
				tmp = tmp->nextHaptor;
			}

			// Removing pattern from scheduling
			// and freeing all the haptors

			cleanList(pattern->activeHaptorList);
			pattern->name             = Null;
			pattern->userParams       = NULL;
			pattern->continuator      = NULL;
			pattern->activeHaptorList = NULL;

			return 0;
		}
	}

	// multi-haptor
	tmp = pattern->activeHaptorList;
	while( tmp )
	{
		TIM_Channel_DutyChanger( statusParams->duty, tmp->id );
		tmp = tmp->nextHaptor;
	}

	return 0;
}

/**
 * @brief Pattern generator **initiator**
 *
 * This function is called by the **User API Module** of HapticLib,
 * when the library user calls hl_startPattern().
 *
 * The first task for the pattern generator **initiator** is to
 * validate the user provided inputs, this cannot be done
 * anywhere else.
 *
 * The pattern initiator then sets up all the status parameters.
 *
 * If the pattern accept user parameters, the status setting can be
 * based on the user provided parameters.
 *
 * The last duty of this function is to put the pattern continuator
 * up for scheduling ( setting
 *    `pattern->continuator = `&`genericContinuator;`), so the
 * next time patternScheduler() will check for active patterns,
 * it will find this pattern.
 *
 * @warning If the **initiator** doesn't set the continuator up for
 *          scheduling, the pattern will never start, and for
 *          HapticLib, the haptor is free to receive other patterns.
 *
 * @param[out] pattern A @ref pattern_desc* holding the running
 *                     instance of the pattern being started.
 *
 * @return uint8_t  The result is returned to indicate success
 * 					or error.
 *
 * @retval 0 Success
 *
 * @retval 1 Error
 *
 */
uint8_t genericPatternGenerator(pattern_desc *pattern)
{
	/*
	 * Create a handy reference for userParameters
	 *                        and statusParameters
	 * Note:
	 * 		- userParams data reside on the user
	 * 					program module. (main.c)
	 * 		- statusParams data reside on SystemDesc
	 * 					variable.
	 */
	genericUserParameters     *userParams = &pattern->userParams->generic;
	genericStatusParameters *statusParams = &pattern->statusParams.generic;


	// User Parameters Validation:
	// Input validation must be made before attempting
	// to use the provided parameters.
	if( userParams == NULL )
	{
#ifdef HL_DEBUG
		send_string("genericPatternGenerator(): expecting optional Parameters, NULL found.\r\n\0");
#endif
		return 1;
	}
	if( userParams->checkParam != rightValue )
	{
#ifdef HL_DEBUG
		send_string("Wrong parameter value: checkParam must be \0");
		send_int( rightValue );
		send_string("\r\n\0");
#endif
			return 1;
	}


	// Status initialization based on user provided parameters...
	if( userParams->increment != bigIncrement )
	{
		statusParams->duty = 555;
	}


	// ...or static status initialization
	statusParams->flag = 0;
	statusParams->duty = 0;


	// Set up for scheduling.
	pattern->continuator = &genericContinuator;

	return 0;
}
