/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file generic.h
 *
 * @brief **Generic Pattern** generator function header [TEMPLATE]
 *
 * [TEMPLATE]
 *
 * This is a template file; copy and rename it as starting point
 * for the implementation of a new Pattern Generator.
 *
 * Follow the structure of this example pattern (code and
 * documentation formats) while implementing a new pattern generator.
 *
 * Pattern specific **constant definitions** can be inserted here.
 *
 * The pattern specific **status parameters** typedef is defined here.
 *
 * The pattern specific **user parameters** typedef is defined here.
 *
 * @note Neither of the two (status or user)  typedef are formally
 * necessary, a super simple pattern generator could even be
 * implemented without the status parameters (but then it could be
 * only externally stopped calling hl_stopPattern()).
 *
 * Please use at least a status Parameter where the pattern progress
 * is stored, to let the **continuator** keep track of itself and
 * decide when to end the pattern and free the haptor(s).
 *
 * @note Remember to use the associated module file too.
 * 		 For the template is:
 * 			- generic.c: Documentation, initiator and continuator.
 * 			- generic.h: Pattern specific definitions, user and
 * 						 status parameters definitions.
 *
 * @note For your pattern they become:
 *          - `mypattern.c`: Documentation, initiator and continuator.
 *          - `mypattern.h`: Pattern specific definitions, user and
 * 						     status parameters definitions
 *
 * [TEMPLATE]
 *
 *
 * Pattern specific **constant definitions** can be inserted here.
 *
 * The pattern specific **status parameters** typedef is defined here.
 *
 * The pattern specific **user parameters** typedef is defined here.
 *
 *
 * @authors Leonardo Guardati
 * @authors Silvio Vallorani
 * @version v0.7
 * @date 2012
 *
 */

/**
 * @brief User Parameter Values for **increment** user parameter.
 *
 * This example show how to define the valid values for
 * a user parameter. The name convention is as follow:
 *
 * \<pattern_name\>\<parameter_name\>
 *
 * For example, the **increment** parameter for the
 * **Generic** pattern accepts only two values:
 *    - smallIncrement: for slow ramps.
 *    - bigIncrements: for fast ramps.
 *
 * This is the enumeration used to hold these two
 * values.
 *
 */
typedef enum genericIncrement {
	smallIncrement = 0,
	bigIncrement   = 1
} genericIncrement;

/**
 * @brief User Parameter Values for **checkParam** user parameter.
 *
 * This example show how to define the valid values for
 * a user parameter. The name convention is as follow:
 *
 * \<pattern_name\>\<parameter_name\>
 *
 * For example, the **checkParam** parameter for the
 * **Generic** pattern accepts only one value:
 *    - rightValue: needed to start the pattern.
 *
 * This is the enumeration used to hold this value.
 *
 */
typedef enum genericCheckParam {
	rightValue = 300000
} genericCheckParam;

 /**
  * @brief Pattern specific **user** parameters type definition.
  *
  * This structure MUST be clearly documented to avoid pitfalls on
  * the generator's use by the application code.
  *
  * The user provided values, must be validated by the (preferably)
  * pattern **initiator** before using them in the pattern logic.
  *
  */
typedef struct genericUserParameters {
	genericIncrement  \
			                   increment; /**< @brief An example
								               user parameter. */
	genericCheckParam \
							   checkParam;/**< @brief Another example
								               user parameter.*/
} genericUserParameters;

/**
 * @brief Pattern specific **status** parameters type definition.
 *
 * At least one parameter should be defined to help the
 * **continuator** keep track of the pattern generation progress.
 *
 * @see Refer to generic.c for a simple working example.
 *
 */
typedef struct genericStatusParameters {
	uint8_t  flag;    /**<@brief a typical status
						  parameter could be a flag. */
	uint16_t duty;    /**< @brief another common status parameter
	 	 	 	 	 	 	can be a `duty` value, used to feed
	 	 	 	 	 	 	the new duty cycle to the PWM and
	 	 	 	 	 	 	together keep track of the progress.*/
} genericStatusParameters;
