/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file hl_patterns.c
 *
 * @brief **Pattern Generator Module** definitions
 *
 * hl_patterns.c is part of the **Pattern Generator Module**
 * of HapticLib.
 *
 * This file lists of all the pattern generators initiators,
 * and uses them to fill the @ref patternMap array.
 *
 * The technique used by HapticLib make it possible to
 * decouple the **User API Module**  and the
 * **Pattern Generator Module**.
 *
 * @see Please refer to the HapticLib [Developer](@ref dev_guide)
 *      documentation page.
 *
 * The **User API Module** only knows the signature of
 * a generic pattern generator function (that is always the same)
 * regardless of what the actual function will be.
 *
 *
 * @authors Leonardo Guardati
 * @authors Silvio Vallorani
 * @version v0.7
 * @date 2012
 *
 */

/**
 * @brief Used to indicate this module is a System one.
 *
 * Declaring this symbol, some internal data structures
 * will be available to the module.
 *
 */
#define  HL_SYSTEM_FILE

#include "hapticLib.h"

extern system_desc SystemDesc;

extern uint8_t  genericPatternGenerator( pattern_desc * );
extern uint8_t     testPatternGenerator( pattern_desc * );
extern uint8_t   impactPatternGenerator( pattern_desc * );
extern uint8_t constantPatternGenerator( pattern_desc * );

/**
 * @brief Pattern Generators Functions Map
 *
 * This array contains the addresses of all the pattern generator
 * initiator functions.
 *
 * The index of the array locate the specific pattern using one of
 * the Pattern Generator Functions Index Names defined in the
 * @ref pattern_name enumeration type definition. (e.g. Test, Impact)
 *
 */
pattern_initiator patternMap[Num_Patterns_Available] = \
		{   NULL,					 \
			genericPatternGenerator,  \
			testPatternGenerator,      \
			impactPatternGenerator,     \
			constantPatternGenerator	 };

/**
 * @brief Updates all the active patterns' progress.
 *
 * patternScheduler() checks for all the patterns configured in
 * the system. If the pattern is running, its continuator is called.
 *
 * patternScheduler() is called by the **Platform Specific** Module
 * ( for example in the STM32VLDISCOVERY platform, SysTick_Handler()
 *  is the caller).
 *
 * @see Please refer to the [Developer Guide](@ref dev_guide) page
 *      to know the internal details of how HapticLib multi-haptor
 *      technique works.
 */
void patternScheduler(void)
{
	uint8_t i=0;

	for( i=0; i < SystemDesc.num_haptors ; i++ )
	{
		if( SystemDesc.patterns[i].continuator )
		{
			if( SystemDesc.patterns[i].continuator( &SystemDesc.patterns[i] ) )
			{
#ifdef HL_DEBUG
				send_string("patternScheduler(): Problem invoking continuator \0");
				send_int( i );
				send_string("\r\n\0");
#endif
			}
		}
	}

}

/**
 * @brief Utility function to clean the pattern list of haptors.
 *
 * This function holds the frequently used code to free the
 * haptor list form the attached pattern.
 *
 * @param[out] haptor The `haptor_desc *` hold by activeHaptorList
 *                    inside `pattern_desc` structure.
 *
 */
void cleanList(haptor_desc *haptor)
{
	haptor_desc *tmp = NULL;

	while( haptor )
	{
		tmp = haptor->nextHaptor;

		haptor->nextHaptor = NULL;
		haptor->activePattern = NULL;

		haptor = tmp;
	}
}

/**
 * @brief Utility function to convert duty-cycles
 *
 * Given the absolute duty-cycle (0-65535) and the haptor
 * this function will return the actual duty cycle honoring
 * min_duty and max_duty values of the haptor.
 *
 * @param[in] duty A `uint16_t` value representing the
 *                 absolute duty cycle.
 *
 * @param[out] haptor A `haptor_desc *` holding the reference
 * 					  to the haptor for which the duty has to
 * 					  be scaled.
 *
 * @return uint16_t the scaled value of the duty cycle.
 *
 */
uint16_t dutyConverter(uint16_t duty, haptor_desc *haptor)
{
	return duty*(haptor->max_duty - haptor->min_duty)/65535 + haptor->min_duty;
}
