/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file hl_patterns.h
 *
 * @brief **Pattern Generator Module** headers
 *
 * hl_patterns.h is part of the **Pattern Generator Module**
 * of HapticLib.
 *
 * In this file there are all the general definitions needed by
 * HapticLib to call a pattern (the code specific to a particular
 * pattern is in its pattern code module).
 *
 * @todo Define Return values constants for pattern generators.
 *
 * @authors Leonardo Guardati
 * @authors Silvio Vallorani
 * @version 0.7
 * @date 2012
 *
 */

#include <stdint.h>
#include <stdlib.h>

#ifdef HL_DEBUG
#include "hl_debug.h"
#endif

/*
 * Platform Specific Hardware Level
 *
 */
#ifdef STM32VLDISCOVERY
#include "hl_STM32VLDISCOVERY.h"
#elif OTHERS
#include "hl_OTHERS.h"
#endif

#include "test.h"
#include "generic.h"
#include "impact.h"
#include "constant.h"

/**
 * @brief Number of patterns known in the System.
 *
 * This is the number of different pattern offered by the library.
 *
 * There could be use cases where System resources (low memory)
 * impose to lower this number; the result is that only the first
 * MAX_PATTERNS patterns are available to the user.
 *
 * @note MAX_PATTERNS does not refer to the maximum number of
 *       pattern running simultaneously, instead is the number
 *       of them that can be loaded in a running application
 *       and then called. ( @ref MAX_HAPTORS is the maximum number
 *       of running pattern instances).
 *
 *
 * @todo Implement a re-mapping mechanism to easily allow the user
 *       the selection of some specific patterns discarding the
 *       others.
 *
 */
#define MAX_PATTERNS 10

/**
 * @brief Index of pattern names
 *
 * This enumeration makes it easy to index arrays with the name
 * of a particular pattern.
 *
 * Previous versions of HapticLib used \#`define` directives for this
 * purpose.
 *
 * The advantage of using an enumeration type definition is code
 * and debug readability.
 *
 */
typedef enum pattern_name {
	Null,

	Generic,
	Test,
	Impact,
	Constant,

	Num_Patterns_Available
} pattern_name;

/**
 * @brief Pattern specific status parameters container.
 *
 * This type definition is useful to increase the readability
 * of the code and to enforce compiler type checking when passing
 * arguments to functions.
 *
 * The status parameters are intended only for the pattern
 * developer. These parameters are not exposed outside the pattern
 * module.
 *
 * @see Please refer to the generic Pattern Template Generator
 *      ( @ref generic.c ) for usage example of the status
 *      parameters.
 *
 */
typedef union status_param {
	genericStatusParameters  generic;  /**< @brief Generic Pattern
												   Status
												   Parameters. */
	testStatusParameters     test;     /**< @brief Test Pattern
											       Status
											       Parameters. */
	impactStatusParameters   impact;   /**< @brief Impact Pattern
												   Status
												   Parameters. */
	constantStatusParameters constant; /**< @brief Constant Pattern
										   		   Status
												   Parameters. */
} status_param;

/**
 * @brief Pattern specific optional user parameters container.
 *
 * This type definition is useful to increase the readability
 * of the code and to enforce compiler type checking when passing
 * parameters to patterns.
 *
 * Usage example of the Impact Pattern:
 * ~~~~~~~~~~~~~~~~
 * 			...
 * 			haptor_desc *myHaptor = hl_configure(24000000,10,1);
 *
 * 			pattern_desc *myImpactPattern;
 *
 * 			user_param myParams;
 *
 * 			myParams.impact.velocity = Fast;
 * 			myParams.impact.material = Rubber;
 *
 * 			myImpactPattern = hl_initPattern(Impact,&myParams);
 *
 * 			hl_addHaptor(myHaptor,myImpactPattern);
 *
 * 			hl_startPattern(myImpactPattern);
 * 			...
 * ~~~~~~~~~~~~~~~~
 *
 */

typedef union user_param {
	impactUserParameters   impact;   /**< @brief Impact Pattern
	                                      User Parameters */
	genericUserParameters  generic;  /**< @brief Generic Pattern
	                                      User Parameters */
	constantUserParameters constant; /**< @brief Constant Pattern
	                                      User Parameters */
} user_param;

struct haptor_desc;

struct pattern_desc;

/**
 * @brief Pattern Generator **Continuator** callback function pointer.
 *
 * Before reading about this function pointer, please read
 * HapticLib [Architecture](@ref hl_architecture) and
 * [Developer](@ref dev_guide) documentation pages.
 *
 * This pointer is used by HapticLib code to transparently call
 * the pattern continuator code of the right pattern.
 *
 * A running pattern instance is active if its descriptor
 * (pattern_desc) holds a valid `pattern_continuator` reference.
 *
 * When hl_startPattern() is called by the user application, the right
 * pattern initiator is called. The initiator, after some validation
 * checks, sets the continuator callback function pointer for the
 * passed pattern_desc.
 *
 */
typedef uint8_t(* pattern_continuator)(struct pattern_desc *);

/**
 * @brief Pattern descriptor
 *
 * The pattern descriptor holds all the relevant information about
 * a pattern being run on a specific haptor.
 *
 */
typedef struct pattern_desc {

	pattern_name	    name;            /**< @brief Running
	                                          Pattern type */
	user_param	       *userParams;      /**< @brief Pointer to the
										      optional user
										      parameters of the
										      pattern. */
	status_param	    statusParams;    /**< @brief Structure
	                                          holding the pattern
	                                          specific status
	                                          informations of the
	                                          generator */
	pattern_continuator continuator;     /**< @brief pattern
	                                          continuator callback
	                                          function pointer. */
	struct haptor_desc *activeHaptorList;/**< @brief Pointer to the
	                                          first haptor_desc
	                                          structure forming the
	                                          list of haptors
	                                          activated by this
	                                          pattern. */
} pattern_desc;

/**
 * @brief Pattern Generation **Initiator** callback function pointer.
 *
 * Before reading about this function pointer, please read
 * HapticLib [Architecture](@ref hl_architecture) and
 * [Developer](@ref dev_guide) documentation pages.
 *
 * This pointer is used by HapticLib code to transparently call
 * the pattern initiator code of the right pattern.
 *
 * HapticLib declares @ref patternMap ; it is an array of
 * `pattern_initiator`s. The index of this array is a
 * @ref pattern_name element. Each element points to the right
 * Pattern Generator's Initiator callback function.
 *
 */
typedef uint8_t(* pattern_initiator)(pattern_desc *);

void patternScheduler(void);
void cleanList(struct haptor_desc *);
uint16_t dutyConverter(uint16_t , struct haptor_desc *);
