// Copyright (c) 2012, Silvio Vallorani <silvio.vallorani@studio.unibo.it>
//
// Permission  to  use,  copy,  modify,  and/or distribute this 
// software  for  any  purpose  with  or  without fee is hereby 
// granted,  provided  that the above copyright notice and this 
// permission notice appear in all copies.
//
// THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS 
// ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL 
// IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO 
// EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, 
// INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES 
// WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS, 
// WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER 
// TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE 
// USE OR PERFORMANCE OF THIS SOFTWARE.
//**
// @file impact.m
//
// Here 256 samples for impactPatterns are generated accordingly to
// 
// Taku Hachisu et al. (2011)
// Pseudo-Haptic Feedback Augmented with Visual and Tactile Vibrations
// 
// Q(t) =  A(v) * e^(-B*t) * sin(2*%pi*f*t)     (1)
// 
// where Amplitude is function of velocity but quasi-linearity implies
// we can semplify in
//
// Q(t) =  A * v * e^(-B*t) * sin(2*%pi*f*t)
//
// Use this formula as Haptic Pattern Sample Generator needs some other
// manipulations:
//
// PS(t) = abs(A) * v * [ 2*e^(-B*t) * (1 + sin(2*%pi*f*t) ]
//
// This approch ensures non negative samples mainteining continuity.
// Samples are also scaled to remain in (1/3) * PMN_LEVELS range (slow).
//
// @author Silvio Vallorani
// @version v0.1
// @date 2012
//
//**

//###### clean ambient
xdel(winsid());
clear;
clc;

//###### Global Parameters
PWM_LEVELS = 65000; // number of PWM levels that can be used in HapticLib

t = ( 0 : 0.00035 : 0.1 );  // ensure impact transitory estinguish
                            // in 256 samples

V1 = 1; V2 = 2; V3 = 3; // Three different impact velocity magnitude are 
                        // defined. 
                        // V1 => SLOW
                        // V2 => NORMALW
                        // V3 => FAST
                        //
                        // Samples must be generated using V1.
                        

//###### ALUMINUM
A_Aluminum = - 300;              A_Aluminum = abs ( A_Aluminum );
B_Aluminum = 90;
F_Aluminum = 300;

S_Aluminum = 1 + sin( 2 * %pi * F_Aluminum * t );
E_Aluminum = 2 * exp( - B_Aluminum * t );

H_Aluminum = A_Aluminum * S_Aluminum .* E_Aluminum;
SCALE_FACTOR = ceil( max( H_Aluminum ) );
H_Aluminum = floor ( H_Aluminum / SCALE_FACTOR * (PWM_LEVELS/3) );
H_Aluminum_V1 = V1 * H_Aluminum;
H_Aluminum_V2 = V2 * H_Aluminum;
H_Aluminum_V3 = V3 * H_Aluminum;

subplot( 3, 3, 7 );
a=gca();
a.tight_limits = 'on';
a.data_bounds = [0,0;t(256),PWM_LEVELS];
a.auto_scale = 'off';
plot( t, H_Aluminum_V1 )
title( 'ALUMINUM @ SLOW')
subplot( 3, 3, 8 );
a=gca();
a.tight_limits = 'on';
a.data_bounds = [0,0;t(256),PWM_LEVELS];
a.auto_scale = 'off';
plot( t, H_Aluminum_V2 )
title( 'ALUMINUM @ NORMAL' )
subplot( 3, 3, 9 );
a=gca();
a.tight_limits = 'on';
a.data_bounds = [0,0;t(256),PWM_LEVELS];
a.auto_scale = 'off';
plot( t, H_Aluminum_V3 )
title( 'ALUMINUM @ FAST' )


//###### RUBBER
A_Rubber = - 240;                 A_Rubber = abs ( A_Rubber );
B_Rubber = 60;
F_Rubber = 30;

S_Rubber = 1 + sin( 2 * %pi * F_Rubber * t );
E_Rubber = 2 * exp( - B_Rubber * t);

H_Rubber = A_Rubber * S_Rubber .* E_Rubber;
H_Rubber = floor ( H_Rubber / SCALE_FACTOR * (PWM_LEVELS/3) );
H_Rubber_V1 = V1 * H_Rubber;
H_Rubber_V2 = V2 * H_Rubber;
H_Rubber_V3 = V3 * H_Rubber;

subplot( 3, 3, 1 );
a=gca();
a.tight_limits = 'on';
a.data_bounds = [0,0;t(256),PWM_LEVELS];
a.auto_scale = 'off';
plot( t, H_Rubber_V1 )
title( 'RUBBER @ SLOW' )
subplot( 3, 3, 2 );
a=gca();
a.tight_limits = 'on';
a.data_bounds = [0,0;t(256),PWM_LEVELS];
a.auto_scale = 'off';
plot( t, H_Rubber_V2 )
title( 'RUBBER @ NORMAL' )
subplot( 3, 3, 3 );
a=gca();
a.tight_limits = 'on';
a.data_bounds = [0,0;t(256),PWM_LEVELS];
a.auto_scale = 'off';
plot( t, H_Rubber_V3 )
title( 'RUBBER @ FAST' )


//###### WOOD
A_Wood = - 150;                   A_Wood = abs ( A_Wood );
B_Wood = 80;
F_Wood = 100;

S_Wood = 1 + sin( 2 * %pi * F_Wood * t );
E_Wood = 2 * exp( - B_Wood * t );

H_Wood = A_Wood * S_Wood .* E_Wood;
H_Wood = floor ( H_Wood / SCALE_FACTOR * (PWM_LEVELS/3) );
H_Wood_V1 = V1 * H_Wood;
H_Wood_V2 = V2 * H_Wood;
H_Wood_V3 = V3 * H_Wood;

subplot( 3, 3, 4 );
a=gca();
a.tight_limits = 'on';
a.data_bounds = [0,0;t(256),PWM_LEVELS];
a.auto_scale = 'off';
plot( t, H_Wood_V1 )
title( 'WOOD @ SLOW' )
subplot( 3, 3, 5 );
a=gca();
a.tight_limits = 'on';
a.data_bounds = [0,0;t(256),PWM_LEVELS];
a.auto_scale = 'off';
plot( t, H_Wood_V2 )
title( 'WOOD @ SLOW' )
subplot( 3, 3, 6 );
a=gca();
a.tight_limits = 'on';
a.data_bounds = [0,0;t(256),PWM_LEVELS];
a.auto_scale = 'off';
plot( t, H_Wood_V3 )
title( 'WOOD @ SLOW' )


//###### WRITE RESULTS ON TXT FILE
fileID = mopen('impact.txt', 'w');
formatPattern = '';
for j=1:25
    formatPattern = formatPattern+'%5d,\t%5d,\t%5d,\t%5d,\t%5d,\t%5d,\t%5d,\t%5d,\t%5d,\t%5d,\t\t\\\n';
end
formatPattern = formatPattern+'%5d,\t%5d,\t%5d,\t%5d,\t%5d,';

mfprintf(fileID, 'const uint8_t rubberImpactPattern[256] = {\t\t\t\t\t\t\t\t\t\t\t\\\n');
mfprintf(fileID, formatPattern, H_Rubber_V1(1:255));
mfprintf(fileID, '\t%5d\t\t\t\t\t\t\t\t\t\t};\n\n',0);

mfprintf(fileID, 'const uint8_t woodImpactPattern[256] = {\t\t\t\t\t\t\t\t\t\t\t\\\n');
mfprintf(fileID, formatPattern, H_Wood_V1(1:255));
mfprintf(fileID, '\t%5d\t\t\t\t\t\t\t\t\t\t};\n\n',0);

mfprintf(fileID, 'const uint8_t aluminumImpactPattern[256] = {\t\t\t\t\t\t\t\t\t\t\\\n');
mfprintf(fileID, formatPattern, H_Aluminum_V1(1:255));
mfprintf(fileID, '\t%5d\t\t\t\t\t\t\t\t\t\t};\n\n',0);
mclose(fileID);

printf('\nText file with Samples generated.\n')

//###### PRINT THE MAX
printf('\nBiggest Sample values exracted from FAST Patterns\n\n')
printf('Biggest Rubber Sample: %5d\n',max(H_Rubber_V3))
printf('Biggest Wood Sample: %5d\n',max(H_Wood_V3))
printf('Biggest Aluminum Sample: %5d\n',max(H_Aluminum_V3))
printf('\n\n')