/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file test.c
 *
 * @brief **Test Pattern** generator function definition
 *
 * Implementation code of the Test Pattern.
 *
 * # Introduction
 *
 * This pattern reproduce two linear ramp from 0% to 100% duty-cycle.
 *
 * # Theory
 *
 * Simple pattern with sawtooth profile.
 *
 * # Optional Parameters
 *
 * No optional parameters.
 *
 * # Usage Examples
 *
 * Here is a typical usage scenario for the Test Pattern:
 *
 * ~~~~~~~~~~~~~~~~
 * 		...
 * 		haptor_desc *myHaptor = hl_configure(24000000,15,1);
 *
 * 		pattern_desc *myTestPattern = hl_initPattern(Test,NULL);
 *
 * 		hl_addHaptor(myHaptor, myTestPattern);
 *
 * 		hl_startPattern(myTestPattern);
 * 		...
 * ~~~~~~~~~~~~~~~~
 *
 * A condensed version of the same code  is the follow:
 * ~~~~~~~~~~~~~~~~
 * 		...
 * 		haptor_desc *myHaptor = hl_configure(24000000,15,1);
 *
 * 		hl_startPattern(hl_addHaptor(myHaptor, hl_initPattern(Test,NULL) ) );
 * 		...
 * ~~~~~~~~~~~~~~~~
 *
 * @note This shortcut is valid and can be used only for
 *       single haptor pattern.
 *
 * # Debugging Details
 *
 * testPatternGenerator doesn't need optional parameter, just pass a
 * NULL.
 * If any are passed, an error occurs and if HL_DEBUG is defined this
 * error will be sent on UART.
 *
 * # Additional Notes
 *
 * No additional notes.
 *
 * @authors Leonardo Guardati
 * @authors Silvio Vallorani
 * @version v0.7
 * @date 2012
 *
 */

#include "hapticLib.h"

/**
 * @brief **Test** Pattern generator **continuator**
 *
 * This function is called back from the SysTick_Handler every
 * [SystemDesc](@ref SystemDesc).samples_delay ms.
 *
 * At every call, this continuator will calculate the next value of
 * PWM duty-cycle.
 *
 * After two positive ramps the pattern is finished and remove itself
 * from the scheduling patterns.
 *
 */
uint8_t testContinuator(pattern_desc *pattern)
{

	/*
	 * Create a handy reference for statusParameters
	 * Note:
	 * 		- userParams data are not needed, so user MUST
	 * 		             pass NULL, and **initiator** check for it.
	 * 		- statusParams data reside on SystemDesc variable.
	 */
	testStatusParameters *statusParams = &pattern->statusParams.test;

	if( statusParams->flag == 0 )
	{
		if( statusParams->duty > 64500 )
		{
			statusParams->flag = 1;
			statusParams->duty = 0 ;
		}
	}
	else
	{
		if( statusParams->duty > 64500 )
		{
			// Ensure finish without vibrations
			TIM_Channel_DutyChanger( 0, pattern->activeHaptorList->id );

			// Removing pattern from scheduling
			// and freeing all the haptors

			cleanList(pattern->activeHaptorList);
			pattern->name             = Null;
			pattern->userParams       = NULL;
			pattern->continuator      = NULL;
			pattern->activeHaptorList = NULL;

			return 0;
		}
	}

	statusParams->duty += 650;

	TIM_Channel_DutyChanger( statusParams->duty, pattern->activeHaptorList->id );

	return 0;
}

/**
* @brief Test Pattern generator **initiator**
 *
 * This function is called by the **User API Module** of HapticLib,
 * when the library user calls hl_sendPattern().
 *
 * The first task for the pattern generator **initiator** is to
 * validate the user provided inputs, this cannot be done
 * anywhere else.
 *
 * This pattern initiator then sets up all the status parameters.
 *
 * If the pattern accept user parameters, the status setting can be
 * based on the user provided parameters.
 *
 * The last duty of this function is to put the pattern continuator
 * up for scheduling ( setting
 *    `pattern->continuator = `&`testContinuator;`), so the
 * next time patternScheduler() will check for active patterns, it will find
 * this pattern.
 *
 * @warning If the **initiator** doesn't set the continuator up for
 *          scheduling, the pattern will never start, and for
 *          HapticLib, the haptor is free to receive other patterns.
 *
 * @note This pattern doesn't need user parameters, and check for
 *       this parameter to be NULL.
 *
 * @param[out] pattern A @ref pattern_desc* holding the reference to
 *                     the running pattern instance to work with.
 *                     Through this reference, the continuator can
 *                     extract the user and status parameters in
 *                     order to execute the haptic pattern.
 *
 * @return uint8_t  The result is returned to indicate success
 * 					or error.
 *
 * @retval 0 Success
 *
 * @retval 1 Error
 *
 */
uint8_t testPatternGenerator(pattern_desc *pattern)
{

	/*
	 * Create a handy reference for userParameters and statusParameters
	 * Note:
	 * 		- userParams data reside on the user program module. (main.c)
	 * 		- statusParams data reside on SystemDesc variable.
	 */
	testStatusParameters *statusParams = &pattern->statusParams.test;

	// No User Parameters check it is NULL
	if( pattern->userParams != NULL )
	{
#ifdef HL_DEBUG
		send_string("Test Pattern generator doesn't need optional \0");
		send_string("parameters. Please pass userParameters=NULL\r\n\0");
#endif
		return 1;
	}

	/* Status initialization */
	statusParams->flag = 0;
	statusParams->duty = 0;

	/* Ready to schedule the pattern */
	pattern->continuator  = &testContinuator;

	return 0;
}
