/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file hl_STM32VLDISCOVERY.c
 *
 * @brief **Platform Specific Module** STM32VLDISCOVERY definitions.
 *
 * This file is part of the **Platform Specific
 * Module** of HapticLib.
 *
 * HapticLib support for the STM32VL-DISCOVERY platform  is
 * documented on the [Architecture page](@ref stm32vldiscovery),
 * please refer to it for additional informations.
 *
 *
 * @author Leonardo Guardati
 * @version 0.7
 * @date 2012
 *
 */

#define HL_SYSTEM_FILE


#include  "hapticLib.h"

extern system_desc SystemDesc;

/**
 * @brief Number of SysTicks to wait for Delay() to return.
 *
 * This is used by Delay() and SysTick_Handler() to create time
 * delays on user application.
 *
 * Delay() set this to a number of tick (by default 1ms) SysTick
 * have to interrupt before it can return.
 *
 */
static __IO uint32_t TimingDelay   = 0;

/**
 * @brief Timer Channels status informations
 *
 * This variable holds the status informations on
 * all the Timers channels present in the system.
 *
 * The information is encoded as follow:
 *  - `channelStatus = 0xFF`: Initial value; means the channels
 *  						  are not configured yet, so they
 *  					      cannot be enabled.
 *  - `channelStatus = 0x00`: The channels are configured and
 *  						  ready to be enabled.
 *
 * The status of the n-th channel is then encoded on the n-th
 * bit of channelStatus.
 *
 * For example, if only channel_1 is enabled, `channelStatus=0x01`
 *
 * If `channelStatus=0x16` ( in binary 0001 0110 ) there are 3
 * channels enabled; channel_2,channel_3 and channel_5.
 *
 * @note The maximum number of active channels is 0xEF: 7 channels.
 *
 */
static uint8_t       channelStatus = 0xFF;

/**
 * @brief Enable Clock control for peripherals needed by HapticLib.
 *
 * For STM32VL-DISCOVERY, here is the standard setting:
 *
 *  - APB1:
 *    + TIM3   (PWM Generator)
 *  - APB2:
 *    + GPIOA:
 *      - PA0:  UserButton
 *      - PA6:  TIM3 Ch1
 *      - PA7:  TIM3 Ch2
 *    + GPIOB:
 *      - PB0:  TIM3 Ch3
 *      - PB1:  TIM3 Ch4
 *    + GPIOC:
 *      - PC8:  User  LED
 *    + AFIO:
 *      - Enable TIM3 access to GPIOs.
 *
 * @note Additionally, only if @ref HL_DEBUG is defined, the
 *       following is set too:
 *  - APB2:
 *    + GPIOB:
 *      - PB10: USART3(TX)
 *      - PB11: USART3(RX)
 *    + GPIOC:
 *      - PC9:  Debug LED
 *    + AFIO:
 *      - Enable USART3 access to GPIOs.
 *
 * Here system debug for core and peripherals are set.
 *
 * @note For system debug (that's not HL_DEBUG HapticLib
 *       Debugging Features), the following configuration
 *       will be setup:
 *       - Debugger connection kept for MCU in:
 *         + SLEEP MODE
 *         + STOP MODE
 *         + STANDBY MODE
 *       - TIM3 counter stopped when Core is halted
 *
 */
void RCC_Configuration(void)
{
#ifdef HL_DEBUG
  RCC_APB1PeriphClockCmd( RCC_APB1Periph_USART3 |       \
		  	  	  	  	  RCC_APB1Periph_TIM3   , ENABLE );
#else
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
#endif
  RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOA |     \
		  	  	  	  	  RCC_APB2Periph_GPIOB |      \
		  	  	  	  	  RCC_APB2Periph_GPIOC |       \
		  	  	  	  	  RCC_APB2Periph_AFIO  , ENABLE );

  DBGMCU_Config( DBGMCU_SLEEP | DBGMCU_STANDBY   |       \
		  	  	 DBGMCU_STOP  | DBGMCU_TIM3_STOP , ENABLE );
}

/**
 * @brief Setup the GPIOs used by HapticLib.
 *
 * For STM32VL-DISCOVERY, the following IO will ALWAYS
 * be configured:
 *
 *  - GPIOA:
 *    + PA0:  UserButton
 *    + PA6:  TIM3 Ch1 (AFIO)
 *    + PA7:  TIM3 Ch2 (AFIO)
 *  - GPIOB:
 *    + PB0:  TIM3 Ch3 (AFIO)
 *    + PB1:  TIM3 Ch4 (AFIO)
 *  - GPIOC:
 *    + PC8:  User  LED
 *
 *  @note Additionally, only if @ref HL_DEBUG is defined,
 *        the following IO will be configured:
 *  - GPIOB:
 *    + PB10: USART3(TX) (AFIO)
 *    + PB11: USART3(RX) (AFIO)
 *  - GPIOC:
 *    + PC9:  Debug LED
 *
 */
void GPIO_Configuration(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

#ifdef HL_DEBUG
	// Setup USART3 TX(PB10)
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	// Setup USART3 RX(PB11)
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	// Setup Debug LED (Green) on PC9
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_Init(GPIOC, &GPIO_InitStructure);

    GPIO_ResetBits(GPIOC,GPIO_Pin_9);
#endif

    // Setup GPIO for TIM3
	// TIM3 Channel_1 and Channel_2
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// TIM3 Channel_3 and Channel_4
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	// Setup User Button on PA0
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING ;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// Setup User LED (BLue) on PC8
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_Init(GPIOC, &GPIO_InitStructure);

    GPIO_ResetBits(GPIOC,GPIO_Pin_8);
}

/**
 * @brief Setup System Timers (TIMx and SysTick).
 *
 * This function prepares the timers used by HapticLib.
 * It must be called before any other timer operation.
 *
 * @warning TIM_Configuration() is called internally by
 *          the library and should not be used directly
 *          by the user.
 *
 * This is what happen:
 *  1. TIM3 timer is setup using `pwm_freq` as parameter.
 *  2. TIM3 Channels are set in PWM Output mode.
 *  3. SysTick is set to **1ms** time resolution.
 *
 * The Timer PWM frequency is set using the parameter `pwm_freq`
 * expressing the frequency in Hertz.
 *
 * This platform operates at 24MHz (CPU, AHP, APB1/2) so the TIM
 * timer can reach this speed too.
 *
 * The SysTick is fixed at 1ms operating frequency, so the time
 * resolution of the application is 1ms.
 *
 * @note The Timer ARR is set to 0xFFFF to allow maximum granularity
 *       on PWM DutyCylces
 *
 *
 * @param[in] pwm_freq  this is a `uint32_t` value. Its value
 * 						represents the PWM frequency in Hz.
 * 						Valid range is:  367 to 24000000.
 *
 * @note Keep in mind that the actual PWM frequency is scaled from
 * 						SystemCoreClock. So only fractions of 24MHz
 * 						can be set.
 * 						![PWM Actual Frequency](@ref PWMActualFreq.png)
 * 						However for low values of pwm_freq you can assume
 * 						the actual frequency set is the same.
 *
 */
void TIM_Configuration(uint32_t pwm_freq)
{
	uint16_t                PrescalerValue = 0;
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
	TIM_OCInitTypeDef       TIM_OCInitStructure;
	

	PrescalerValue = (uint16_t) ( SystemCoreClock / pwm_freq );

	// TIM3 Time base configuration

  	TIM_TimeBaseStructure.TIM_Period = 0xFFFF;
  	TIM_TimeBaseStructure.TIM_Prescaler = PrescalerValue;
  	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

  	/* Common Timer Channel setup. */
  	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  	TIM_OCInitStructure.TIM_Pulse = 0;
  	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  	// PWM1 Mode configuration for all the 4 channels.
  	TIM_OC1Init(TIM3, &TIM_OCInitStructure);
  	TIM_OC2Init(TIM3, &TIM_OCInitStructure);
  	TIM_OC3Init(TIM3, &TIM_OCInitStructure);
  	TIM_OC4Init(TIM3, &TIM_OCInitStructure);

  	TIM_ARRPreloadConfig(TIM3, ENABLE);

	TIM_Cmd(TIM3, ENABLE);

	// System Timer has 1ms time resolution.
	if ( SysTick_Config(SystemCoreClock / 1000) )
    		while (1);

	// Mark the channels as configured.
	channelStatus = 0x00;

}

/**
 * @brief Single timer channel activation.
 *
 * TIM_Channel_Enable() activates the single channel passed as
 * argument.
 *
 * @warning TIM_Channel_Enable() is called internally by
 *          the library and should not be used directly
 *          by the user.
 *
 * Before activating the channel, it is checked that the channels
 * are configured ( `channelStatus != 0xFF` ) and that the channel
 * passed is not already active.
 *
 * @note After activation the channel's Duty Cycle is cleared to 0.
 *
 * @todo Clarify the **User API** <--> **Platform Specific**
 * 		 interface; this logic may need to be changed.
 *
 * @param[in] channel  this is a `uint8_t` value. Its value
 * 					   represents the channel to be activated.
 *
 * @return uint8_t  The result is returned to indicate success
 * 					or error.
 *
 * @retval 0 Success
 * @retval 1 Error
 *
 */
uint8_t TIM_Channel_Enable(uint8_t channel)
{
	// Sanity check to avoid enable before configuration.
	if( channelStatus == 0xFF )
	{
#ifdef HL_DEBUG
		send_string("TIM_Channel_Enable(): \0");
		send_int( channel );
		send_string(". Channel not yet configured!\r\n\0");
#endif
		return 1;
	}

	// Check to avoid enabling an already enabled channel.
	if( ( channelStatus >> ( channel-1 ) ) & 0x01  )
	{
#ifdef HL_DEBUG
		send_string("TIM_Channel_Enable(): \0");
		send_int( channel );
		send_string(". Channel already enabled!\r\n\0");
#endif
		return 1;
	}

	// Enable the channel and set to 0 PWM duty cycle.
	switch (channel) {
	case 1:
		TIM_OC1PreloadConfig(TIM3, TIM_OCPreload_Enable);
		TIM_SetCompare1(TIM3, 0);
		channelStatus |= 0x01;
		break;
	case 2:
		TIM_OC2PreloadConfig(TIM3, TIM_OCPreload_Enable);
		TIM_SetCompare2(TIM3, 0);
		channelStatus |= 0x02;
		break;
	case 3:
		TIM_OC3PreloadConfig(TIM3, TIM_OCPreload_Enable);
		TIM_SetCompare3(TIM3, 0);
		channelStatus |= 0x04;
		break;
	case 4:
		TIM_OC4PreloadConfig(TIM3, TIM_OCPreload_Enable);
		TIM_SetCompare4(TIM3, 0);
		channelStatus |= 0x08;
		break;
	default:
#ifdef HL_DEBUG
		send_string("TIM_Channel_Enable(): \0");
		send_int( channel );
		send_string(". Channel not available!\r\n\0");
#endif
		return 1;
	}
	return 0;
}

/**
 * @brief Single timer channel duty cycle update.
 *
 * TIM_Channel_DutyChanger() uses `new_duty` to update
 * the `channel`-th channel.
 *
 * @warning TIM_Channel_DutyChanger() is called internally
 * by the library and should not be used directly by the user.
 *
 * @todo Clarify the **User API** <--> **Platform Specific**
 * 		 interface; this logic may need to be changed.
 *
 * @param[in] new_duty  It is a `uint16_t` value.
 * 						It is the new duty cycle expressed as
 * 						fraction of 0xFFFF.	The range is from
 * 						0x0000 (0%) to 0xFFFF (100%).
 * 						The resulting duty cycle resolution is 1.5‰.
 * @param[in] channel   It is a `uint8_t` value.
 * 						It is the ID of the channel whose PWM control
 * 						signal is going to be updated.
 *
 * @return uint8_t  The result is returned to indicate success
 * 					or error.
 *
 * @retval 0 Success
 * @retval 1 Error
 *
 */
uint8_t TIM_Channel_DutyChanger(uint16_t new_duty, uint8_t channel)
{
	// Sanity check to avoid enable before configuration.
	if( channelStatus == 0xFF )
	{
#ifdef HL_DEBUG
		send_string("TIM_Channel_DutyChanger(): Channel \0");
		send_int( channel );
		send_string(" not yet configured!\r\n\0");
#endif
		return 1;
	}

	// Check if the channel is enabled.
	if( ( ( channelStatus >> ( channel-1 ) ) & 0x01 ) == 0)
	{
#ifdef HL_DEBUG
		send_string("TIM_Channel_DutyChanger(): Channel \0");
		send_int( channel );
		send_string(" must be enabled before using it!\r\n\0");
#endif
		return 1;
	}

	if( (channel > MAX_HAPTORS) || (channel < 1) )
	{
#ifdef HL_DEBUG
		send_string("TIM_Channel_DutyChanger(): Channel \0");
		send_int( channel );
		send_string(" not available.\r\n\0");
#endif
		return 1;

	}

	// Update the new duty cycle for the given haptorID.
	switch (channel) {
	case 1:
		TIM_SetCompare1(TIM3, new_duty);
	  	break;
	case 2:
		TIM_SetCompare2(TIM3, new_duty);
	  	break;
	case 3:
		TIM_SetCompare3(TIM3, new_duty);
	  	break;
	case 4:
		TIM_SetCompare4(TIM3, new_duty);
	  	break;
	default:
		return 1;
	}

	return 0;
}

#ifdef HL_DEBUG
/**
 * @brief **DEBUG function** to setup USART3 for debugging messages.
 *
 * USART_Configuration() sets up the Serial link to display
 * debugging messages if **HapticLib Debugging Features**
 * are enabled. (see the note below)
 *
 * @warning USART_Configuration() is called internally by
 *          the library and should not be used directly by
 *          the user.
 *
 * @note This function is compiled only if
 *       [HL_DEBUG](@ref HL_DEBUG) is defined.
 *
 * The USART3 is configured as follow:
 *       - BaudRate = 115200 baud
 *       - Word Length = 8 Bits
 *       - One Stop Bit
 *       - No parity
 *       - Hardware flow control disabled (RTS and CTS signals)
 *       - Receive and transmit enabled
 *
 * @note The Pins to use for the USART are:
 * 			- TX: PB10
 * 			- RX: PB11
 *
 */
void USART_Configuration(void)
{
	USART_InitTypeDef USART_InitStructure;

	USART_InitStructure.USART_BaudRate = 115200;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	USART_Init(USART3, &USART_InitStructure);
	USART_Cmd(USART3, ENABLE);
}

/**
 * @brief **DEBUG function** to send a byte to the USART3 link.
 *
 * send_char() uses the USART3 peripheral to send the passed `ch`
 * across the serial link.
 *
 * @warning send_char() is called internally by
 *          the library and should not be used directly by
 *          the user.
 *
 * @note This function is compiled only if
 *       [HL_DEBUG](@ref HL_DEBUG) is defined.
 *
 * @param [in] ch The character to send across the serial link.
 *
 * @return The same character sent.
 *
 */
uint8_t send_char(uint8_t ch)
{
  USART_SendData(USART3, ch);

  /* Loop until the end of transmission */
  while (USART_GetFlagStatus(USART3, USART_FLAG_TC) == RESET)
  {}

  return ch;
}
#endif

/**
 * @brief Delay function to wait `nTime` milliseconds.
 *
 * Delay() sets the global @ref TimingDelay variable to the
 * number of times SysTick must decrement it until it reaches
 * zero, then Delay() returns.
 *
 * SysTick generates an interrupt every 1ms.
 *
 * @note Delay sleeps while waiting for SysTick.
 *
 * @param [in] nTime The number of tick to wait. ( 1 tick = 1 ms)
 *
 */
void Delay(__IO uint32_t nTime)
{ 
  TimingDelay = nTime;

  while(TimingDelay != 0) { __WFE(); } ;
}

/**
 * @brief SysTick ISR override.
 *
 * At every SysTick Timer event (1ms), SysTick_Handler() is called.
 *
 * SysTick_Handler() does 2 things:
 *  - Decrements @ref TimingDelay used by Delay()
 *  - Calls the **User API**'s **Pattern Scheduler** to update the
 *    active pattern's PWMs.
 *
 *  @see To know the internal details of how HapticLib multi-haptor
 *       technique works, please refer to the documentation on the
 *       [Developer Guide](@ref dev_guide) page.
 *
 */
void SysTick_Handler(void)
{
	static __IO uint32_t SamplesDelay = 0;

	if( TimingDelay != 0x00 )
		TimingDelay--;

	if( SamplesDelay != 0x00 )
		SamplesDelay--;
	else
	{
		patternScheduler();

		SamplesDelay = SystemDesc.samples_delay;
	}
}
